/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* acquire.c - gather data for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
acquire_data (void)
{
	int i;
	short *sample16;
	double *pwind = (double *) hex_coefficients;
	double *pdata = fft_in;

	if (overlap_mode)
	{
		if (stereo_channel == 0)
		/* Mono selected ... */
		{
			sample16 = (short *) buffer[process_buffer1] + len2;
			for (i = len1; i; i--, sample16++)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

			sample16 = (short *) buffer[process_buffer2];
			for (i = len2; i; i--, sample16++)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

		}														/* end stereo_channel == 0 */

		else if (stereo_channel == 1)
		/* Left channel selected ... */
		{
			sample16 = (short *) buffer[process_buffer1] + 2 * len2;
			for (i = len1; i; i--, sample16 += 2)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

			sample16 = (short *) buffer[process_buffer2];
			for (i = len2; i; i--, sample16 += 2)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

		}														/* end stereo_channel == 1 */

		else
		/* stereo_channel == 2 */
		/* Right channel selected ... */
		{
			sample16 = (short *) buffer[process_buffer1] + 2 * len2 + 1;
			for (i = len1; i; i--, sample16 += 2)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

			sample16 = (short *) buffer[process_buffer2] + 1;
			for (i = len2; i; i--, sample16 += 2)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

		}														/* end stereo_channel == 2 */
	}															/* end overlap */

	else
	/* No overlap */
	{
		sample16 = (short *) buffer[0];

		if (stereo_channel == 0)
		/* Mono selected ... */
			for (i = MAX_LEN; i; i--, sample16++)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

		else
		/* stereo_channel == 1 || stereo_channel == 2 */
		/* Stereo left or right selected ... */
		{
			if (stereo_channel == 2)
				sample16++;

			for (i = MAX_LEN; i; i--, sample16 += 2)
				*pdata++ = (*sample16 - dc_offset) * (*pwind++);

		}
	}															/* end no overlap */
}																/* end acquire_data */

void
get_next_block (void)
{
	if (overlap_mode)
	{
		if (len1 < base_offset || len1 == MAX_LEN)
		/* New block required for next transform ... */
		{
			record_block (buffer[queue_buffer]);
			queue_buffer = !queue_buffer;
		}

	/* Adjust the offsets and lengths ... */
		len1 -= base_offset;

		if ((len2 += base_offset) >= MAX_LEN)
		/* Wrap into the 'next' buffer ... */
		{
			len1 += MAX_LEN;
			len2 -= MAX_LEN;
			process_buffer1 = !process_buffer1;
			process_buffer2 = !process_buffer2;
		}
	}															/* end overlap */

	else
	/* No overlap */
		record_block (buffer[0]);

}																/* end get_next_block */

void
snooze (void)
{
	struct timeval tv = { 0, frame_period };

	select (0, 0, 0, 0, &tv);

}																/* end snooze */

/* vi: set ts=2: */
