/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* border.c - borders etc for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
graph_border (void)
{
	int x = WINDOW_LEFT + XSTEP;

	XDrawRectangle (display, shadow, border_gc,
									WINDOW_LEFT - 1, WINDOW_TOP - 1, WIDE + 1, HIGH);

/* The x-axis ticks ... */
	for (; x < WINDOW_RIGHT + 1; x += XSTEP)
		XDrawLine (display, shadow, border_gc,
							 x, WINDOW_BOTTOM + 1, x, WINDOW_BOTTOM + 5);

}																/* end graph_border */

void
graph_background (void)
{
	XFillRectangle (display, shadow, gback_gc,
									WINDOW_LEFT, WINDOW_TOP, WIDE, HIGH - 1);

}																/* end graph_background */

void
graticule_vertical (void)
{
	int x = WINDOW_LEFT + XSTEP;

	for (; x < WINDOW_RIGHT; x += XSTEP)
		XDrawLine (display, shadow, grat_gc, x, WINDOW_BOTTOM - 1, x, WINDOW_TOP);

}																/* end graticule_vertical */

void
graticule_horizontal (void)
{
	int i;

	for (i = 1; i < (log_top - log_base) / 10; i++)
	{
		int y = WINDOW_TOP + rint (i * y_scale) - 1;

		XDrawLine (display, shadow, grat_gc, WINDOW_LEFT, y, WINDOW_RIGHT, y);
	}
}																/* end graticule_horizontal */

void
peak_string (void)
{
	XDrawString (display, shadow, text_gc, PKX, PKY, "Peak        dB", 14);
	XDrawString (display, shadow, text_gc, PKX + 23 * FONT_WIDTH, PKY, "Hz", 2);

}																/* end peak_string */

void
init_header (void)
{
/* First the labels */
	XDrawString (display, shadow, text_gc, SRX, SRY, "Sampling Hz: ", 13);
	XDrawString (display, shadow, text_gc, OVX, OVY, "Overlap: ", 9);
	XDrawString (display, shadow, text_gc, FSX, FSY, "Frames/sec: ", 12);
	XDrawString (display, shadow, text_gc, DCX, DCY, "DC Offset: ", 11);
	XDrawString (display, shadow, text_gc, AVX, AVY, "Averaging %: ", 13);
	XDrawString (display, shadow, text_gc, SMX, SMY, "Stereo Mode: ", 13);
	XDrawString (display, shadow, text_gc, ZMX, ZMY, "X Zoom: ", 8);
	XDrawString (display, shadow, text_gc, FRX, FRY, "Freeze: ", 8);
	XDrawString (display, shadow, text_gc, RAX, RAY, "Rainbow: ", 9);

/* Now the values */
	update_sample_rate ();
	update_overlap ();
	update_frame_rate ();
	update_dc_offset ();
	update_averaging ();
	update_stereo ();
	update_zoom ();
	update_freeze ();
	update_rainbow ();

}																/* end init_header */

/* vi: set ts=2: */
