/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* callback.c - callback functions for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
do_toggle_averaging (int dummy)
{
	if ((averaging_mode = !averaging_mode))
		reset_averaging ();

	update_averaging ();

}																/* end do_toggle_averaging */

void
do_averaging_down (int dummy)
{
	averaging_factor -= AVERSTEP;
	reset_averaging ();
	update_averaging ();

}																/* end do_averaging_down */

void
do_averaging_up (int dummy)
{
	averaging_factor += AVERSTEP;
	reset_averaging ();
	update_averaging ();

}																/* end do_averaging_up */

void
do_set_averaging (int param)
{
	averaging_factor = param;
	reset_averaging ();
	update_averaging ();
}																/* end do_set_averaging */

void
do_toggle_freeze (int dummy)
{
	freeze_mode = !freeze_mode;
	update_freeze ();

	if (freeze_mode)
		reset_soundcard ();

	else
	{
		if (averaging_mode)
			reset_averaging ();

		if (overlap_mode)
			reset_overlap ();

		init_record ();
	}
}																/* end do_toggle_freeze */

void
do_toggle_graticule (int dummy)
{
	graticule_mode = !graticule_mode;
	graticule_horizontal ();
	graticule_vertical ();

}																/* end do_toggle_graticule */

void
do_toggle_peak (int dummy)
{
	if ((peak_mode = !peak_mode))
	{
		peak_string ();

		last_peak_f = RESETF;
		last_peak_dB = RESETDB;

		if (freeze_mode)
			first_peak = 1;
	}

	else
		XFillRectangle (display, shadow, back_gc,
										PKX, PKY - FONT_HEIGHT, 25 * FONT_WIDTH, FONT_HEIGHT);

}																/* end do_toggle_peak */

void
do_toggle_rainbow (int dummy)
{

	rainbow_mode = !rainbow_mode;
	graph_background ();

	if (graticule_mode)
	{
		graticule_horizontal ();
		graticule_vertical ();
	}

	reset_last_y ();
	update_rainbow ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_toggle_rainbow */

void
do_toggle_overlap (int dummy)
{
	overlap_mode = !overlap_mode;
	init_frame_rate ();
	update_frame_rate ();
	update_overlap ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_toggle_overlap */

void
do_overlap_down (int dummy)
{
	overlap_factor--;
	init_frame_rate ();
	update_frame_rate ();
	update_overlap ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_overlap_down */

void
do_overlap_up (int dummy)
{
	overlap_factor++;
	init_frame_rate ();
	update_frame_rate ();
	update_overlap ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_overlap_up */

void
do_set_overlap (int param)
{
	overlap_factor = param;
	init_frame_rate ();
	update_frame_rate ();
	update_overlap ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_set_overlap */

void
do_rate_up (int dummy)
{
	rate_index++;
	init_frame_rate ();
	update_frame_rate ();
	frequency_scale ();
	update_sample_rate ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_rate_up */

void
do_rate_down (int dummy)
{
	rate_index--;
	init_frame_rate ();
	update_frame_rate ();
	frequency_scale ();
	update_sample_rate ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_rate_down */

void
do_set_rate (int param)
{
	rate_index = param;
	init_frame_rate ();
	update_frame_rate ();
	frequency_scale ();
	update_sample_rate ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();
}																/* end do_set_rate */

void
do_stereo_up (int dummy)
{
	if (++stereo_channel > STEREOMAX)
		stereo_channel = STEREOMIN;

	update_stereo ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_stereo_up */

void
do_stereo_down (int dummy)
{
	if (--stereo_channel < STEREOMIN)
		stereo_channel = STEREOMAX;

	update_stereo ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_stereo_down */

void
do_set_stereo (int param)
{
	stereo_channel = param;
	update_stereo ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_set_stereo */

void
do_callibrate_dc (int dummy)
{
	callibrate_soundcard ();
	update_dc_offset ();
	reset_soundcard ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

	init_record ();

}																/* end do_callibrate_dc */

void
do_quit (int dummy)
{
	stop_threads ();
	save_ini_file ();
	fftw_destroy_plan (the_plan);

	if (sound_set)
		reset_soundcard ();

	cleanup_graphics ();
	exit (EXIT_SUCCESS);

}																/* end do_quit */

/* vi: set ts=2: */
