/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* dialog.c - dialog for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

static pthread_t dialog_thread;

/* Common to main and dialog threads ... protected by mutex ... */
static volatile void (*my_callback) (int) = NULL;
static volatile int my_param;
static pthread_mutex_t data_mutex = PTHREAD_MUTEX_INITIALIZER;

void
check_dialog (void)
/* Only called from main thread ... */
{
	pthread_mutex_lock (&data_mutex);

	if (my_callback)
	/* Call the chosen function ... */
	{
		my_callback (my_param);
		my_callback = NULL;					/* Make them wait ... */
	}

	pthread_mutex_unlock (&data_mutex);

}																/* end check_dialog */

void
start_threads (void)
/* Only called from main thread ... */
{
	if (pthread_create (&dialog_thread, NULL, perform_dialog, NULL))
		error ("Cannot create dialog thread.");

}																/* end start_dialog */

void
stop_threads (void)
/* Only called from main thread ... */
{
	pthread_join (dialog_thread, NULL);

}																/* end stop_dialog */

void *
perform_dialog (void *dummy)
/* Dialog thread ... */
{
	volatile void (*this_callback) (int);

	dialog_signals ();

	while (1)
	{
		pthread_mutex_lock (&data_mutex);
		this_callback = my_callback;
		pthread_mutex_unlock (&data_mutex);

		if (this_callback == NULL)
			check_events ();

		else
			snooze ();

	}															/* end while */
}																/* end perform_dialog */

void
dispatch_dialog (void (*this_callback) (int), int this_param)
/* Only called from dialog thread ... */
{
	pthread_mutex_lock (&data_mutex);
	my_callback = (volatile void *) this_callback;
	my_param = (volatile int) this_param;
	pthread_mutex_unlock (&data_mutex);

	if (this_callback == do_quit)
		pthread_exit (NULL);

}																/* end dispatch_dialog */

/* vi: set ts=2: */
