/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* drawgraph.c - draw `xspect' graph by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
draw_graph (void)
{
	int x, y;											/* Used for drawing */
	int *py = last_y;							/* Last graph y array */
	double *pv = power_val + first_bin;

	if (rainbow_mode)
	/* Rainbow graph */
	{
		unsigned long int *ulp = rainbow + first_bin;

		if (num_bins == WIDE)
		/* Lines */
		{
			for (x = WINDOW_LEFT; x <= WINDOW_RIGHT; x++, pv++, py++, ulp++)
			{
				if (*pv == 0.0)
					y = WINDOW_BOTTOM;

				else
				/* Non-zero power */
				{
					y = WINDOW_TOP - rint (log10 (*pv) * y_scale + y_offset);

					if (y > WINDOW_BOTTOM)
						y = WINDOW_BOTTOM;

					else if (y < WINDOW_TOP)
						y = WINDOW_TOP;

				}												/* end non-zero power */

				if (y != *py)
				{
					if (y > *py)
					/* Undraw the top bit ... */
					{
						XSetForeground (display, rain_gc, *ulp);
						XDrawLine (display, shadow, rain_gc, x, y - 1, x, *py);
					}

					else
					/* Draw the excess ... */
					{
						XSetForeground (display, rain_gc, *ulp);
						XDrawLine (display, shadow, rain_gc, x, *py - 1, x, y);
					}

					*py = y;

				}												/* end if (y != *py) */
			}													/* end for */
		}														/* end lines */

		else
		/* Rectangles */
		{
			int count, width = WIDE / num_bins;

			for (x = WINDOW_LEFT, count = 0;
					 count < num_bins; count++, x += width, pv++, py++, ulp++)
			{
				if (*pv == 0.0)
					y = WINDOW_BOTTOM;

				else
				/* Non-zero power */
				{
					y = WINDOW_TOP - rint (log10 (*pv) * y_scale + y_offset);

					if (y > WINDOW_BOTTOM)
						y = WINDOW_BOTTOM;

					else if (y < WINDOW_TOP)
						y = WINDOW_TOP;

				}												/* end non-zero power */

				if (y != *py)
				{
					if (y > *py)
					/* Undraw the top bit ... */
					{
						XSetForeground (display, rain_gc, *ulp);
						XFillRectangle (display, shadow, rain_gc, x, *py, width, y - *py);
					}

					else
					/* Draw the excess ... */
					{
						XSetForeground (display, rain_gc, *ulp);
						XFillRectangle (display, shadow, rain_gc, x, y, width, *py - y);
					}

					*py = y;

				}												/* end if (y != *py) */
			}													/* end for */
		}														/* end rectangles */
	}															/* end rainbow graph */

	else
	/* Mono graph */
	{
		if (num_bins == WIDE)
		/* Lines */
		{
			int nsegs = 0;
			XSegment *xsp = graph_segs;

			for (x = WINDOW_LEFT; x <= WINDOW_RIGHT; x++, pv++, py++)
			{
				if (*pv == 0.0)
					y = WINDOW_BOTTOM;

				else
				/* Non-zero power */
				{
					y = WINDOW_TOP - rint (log10 (*pv) * y_scale + y_offset);

					if (y > WINDOW_BOTTOM)
						y = WINDOW_BOTTOM;

					else if (y < WINDOW_TOP)
						y = WINDOW_TOP;

				}												/* end non-zero power */

				if (y != *py)
				{
					if (y > *py)
					/* Undraw the top bit ... */
					{
						xsp->y1 = y - 1;
						xsp->y2 = *py;
					}

					else
					/* Draw the excess ... */
					{
						xsp->y1 = *py - 1;
						xsp->y2 = y;
					}

					xsp->x1 = xsp->x2 = x;
					xsp++;
					nsegs++;
					*py = y;

				}												/* end if (y != *py) */
			}													/* end for */

			if (nsegs)
			/* Draw all segments at once ... */
				XDrawSegments (display, shadow, graph_gc, graph_segs, nsegs);

		}														/* end lines */

		else
		/* Rectangles */
		{
			int nrects = 0;
			XRectangle *xrp = graph_rects;
			int count, width = WIDE / num_bins;

			for (x = WINDOW_LEFT, count = 0;
					 count < num_bins; count++, x += width, pv++, py++)
			{
				if (*pv == 0.0)
					y = WINDOW_BOTTOM;

				else
				/* Non-zero power */
				{
					y = WINDOW_TOP - rint (log10 (*pv) * y_scale + y_offset);

					if (y > WINDOW_BOTTOM)
						y = WINDOW_BOTTOM;

					else if (y < WINDOW_TOP)
						y = WINDOW_TOP;

				}												/* end non-zero power */

				if (y != *py)
				{
					if (y > *py)
					/* Undraw the top bit ... */
					{
						xrp->y = *py;
						xrp->height = y - *py;
					}

					else
					/* Draw the excess ... */
					{
						xrp->y = y;
						xrp->height = *py - y;
					}

					xrp->x = x;
					xrp->width = width;
					xrp++;
					nrects++;
					*py = y;

				}												/* end if (y != *py) */
			}													/* end for */

			if (nrects)
			/* Draw all rectangles at once ... */
				XFillRectangles (display, shadow, graph_gc, graph_rects, nrects);

		}														/* end rectangles */
	}															/* end mono graph */
}																/* end draw_graph */

/* vi: set ts=2: */
