/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* extern.h - prototypes & globals for `xspect' by Gordon Miller 2007 */

/* acquire.c */
void acquire_data (void);
void get_next_block (void);
void snooze (void);

/* border.c */
void graph_border (void);
void graph_background (void);
void graticule_vertical (void);
void graticule_horizontal (void);
void peak_string (void);
void init_header (void);

/* callback.c */
void do_toggle_averaging (int dummy);
void do_averaging_down (int dummy);
void do_averaging_up (int dummy);
void do_set_averaging (int param);
void do_toggle_freeze (int dummy);
void do_toggle_graticule (int dummy);
void do_toggle_peak (int dummy);
void do_toggle_rainbow (int dummy);
void do_toggle_overlap (int dummy);
void do_overlap_down (int dummy);
void do_overlap_up (int dummy);
void do_set_overlap (int param);
void do_rate_up (int dummy);
void do_rate_down (int dummy);
void do_set_rate (int param);
void do_stereo_up (int dummy);
void do_stereo_down (int dummy);
void do_set_stereo (int param);
void do_callibrate_dc (int dummy);
void do_quit (int dummy);

/* dialog.c */
void check_dialog (void);
void start_threads (void);
void stop_threads (void);
void *perform_dialog (void *dummy);
void dispatch_dialog (void (*this_callback) (int), int this_param);

/* dimmer.c */
int channel_is (int param);
int rate_is (int param);
int averaging_is (int param);
int averaging_off (int dummy);
int overlap_is (int param);
int overlap_off (int dummy);
int x_zoom_is (int param);
int x_zoom_off (int dummy);
int x_zoom_max (int dummy);
int x_zoom_start (int dummy);
int x_zoom_end (int dummy);
int y_zoom_off (int dummy);
int y_zoom_max (int dummy);
int y_zoom_top (int dummy);
int y_zoom_bottom (int dummy);
int freeze_on (int dummy);

/* drawgraph.c */
void draw_graph (void);

/* fftw.c */
void init_wisdom (void);
void get_power (void);

/* global.c */

/* help.c */
void show_help_window (int dummy);
void close_help_window (void);
void show_about_window (int dummy);
void close_about_window (void);

/* inifile.c */
void parse_ini_file (void);
void save_ini_file (void);

/* longopts.c */
void parse_command_line (int argc, char *argv[]);
void show_help (void);
void show_version (void);
void show_error (void);

/* main.c */
int main (int argc, char *argv[]);

/* motion.c */
void do_vertical_down (int dummy);
void do_vertical_up (int dummy);
void do_vertical_more (int dummy);
void do_vertical_less (int dummy);
void do_vertical_top (int dummy);
void do_vertical_bottom (int dummy);
void do_vertical_full (int dummy);
void do_horizontal_start (int dummy);
void do_horizontal_end (int dummy);
void do_horizontal_full (int dummy);
void do_horizontal_more (int dummy);
void do_horizontal_less (int dummy);
void do_horizontal_left (int dummy);
void do_horizontal_right (int dummy);
void do_horizontal_zoom (int param);

/* params.c */
void check_parameters (void);

/* peak.c */
void draw_peak (void);

/* rainbow.c */

/* reset.c */
void reset_last_y (void);
void reset_averaging (void);
void reset_overlap (void);
void reset_zoom (void);

/* scales.c */
void setup_logscales (void);
void frequency_scale (void);
void power_scale (void);

/* setup.c */
void initialize (int argc, char *argv[]);
void setup_buffers (void);
void error (char *str);

/* signal.c */
void main_signals (void);
void dialog_signals (void);
void termination_handler (int signum);
void suspend_handler (int signum);
void resume_handler (int signum);

/* sound.c */
void init_record (void);
void reset_soundcard (void);
void callibrate_soundcard (void);
void record_block (void *buffer);
void init_frame_rate (void);

/* update.c */
void update_sample_rate (void);
void update_overlap (void);
void update_dc_offset (void);
void update_stereo (void);
void update_averaging (void);
void update_zoom (void);
void update_freeze (void);
void update_rainbow (void);
void update_frame_rate (void);

/* window.c */

/* xevent.c */
void check_events (void);

/* xgraphics.c */
void setup_graphics (void);
void cleanup_graphics (void);
void erase_canvas (void);
void setup_palette (void);
unsigned long draw_setgc_fore (GC gc, Rgb * colour);
void draw_setgc_back (GC gc, unsigned long pixel);

/* xkeyinput.c */
void process_key (int c);

/* xmenu.c */
void draw_menu (int menu_number, int x, int y);
void perform_menu (int menu_number, XEvent * eventp);
void update_menu (int menu_number, XEvent * eventp);
void highlight (Menu * mp, int entry);
void unhighlight (Menu * mp, int entry);
void arrow (Menu * mp, int entry, GC first, GC second, GC third);
void kill_menus (int start_menu);
void kill_all_menus (void);

/* Various globals */

extern char alsa_device[];			/* Space for the ALSA device string */

extern volatile int quit_signal;	/* Signal to quit received */
extern volatile int suspend_signal;	/* Signal to suspend received */
extern volatile int resume_signal;	/* Signal to resume received */

extern int sound_set;						/* Soundcard set */
extern int graph_set;						/* Graphics set */

extern int dc_offset;						/* DC offset for data */
extern int allow_rates[];				/* Array of allowable sample rates */
extern int rate_index;					/* Index in array of allowable sample rates */
extern int stereo_channel;			/* Which channel(s) to show */
extern char *mode_names[];			/* Stereo mode names */

/* Data for display updates */

extern int *last_y;							/* Memory for the previous display location */
extern double *power_val;				/* Current power value */
extern double *last_power;			/* Last power value for averaging mode */
extern double y_scale;					/* Scaling factor */
extern double y_offset;					/* Zero dB offset from log_top */
extern int log_base;						/* Base of log y scale (in dB) */
extern int log_top;							/* Max. of log y scale (in dB) */
extern int peak_mode;						/* Flag for peak power */
extern double last_peak_dB;			/* Last peak power dB */
extern double last_peak_f;			/* Last peak frequency Hz */
extern int freeze_mode;					/* Flag for freeze-frame mode */
extern int averaging_mode;			/* Flag for averaging mode */
extern int averaging_factor;		/* Extent of averaging (%) */
extern double averaging_alpha;	/* How much this frame */
extern double averaging_beta;		/* How much last frame */
extern int overlap_mode;				/* Flag for input block overlap mode */
extern int overlap_factor;			/* Extent of overlap */
extern double frame_rate;				/* Frames per second */
extern int frame_period;				/* Period in uS of frame rate */

extern int help_on;							/* Help window shown */
extern int about_on;						/* About window shown */
extern int scr_list_len;				/* How many active menu windows */
extern int screen_list[];				/* List of active menu window numbers */
extern Menu menus[];						/* Main list of menus */

/* Data for the FFT routines */
extern fftw_plan the_plan;			/* Used for FFT plan */
extern double *fft_in;					/* Used for transforms */
extern double *fft_out;					/* Output of FFT transform */

extern unsigned long long int hex_coefficients[];	/* The windowing coeffs */

/* Buffers for sampling new data */

extern void *buffer[];					/* Buffers for gathering data */
extern int queue_buffer;				/* Next buffer to be queued */
extern int process_buffer1;			/* 1st overlap buffer to be processed */
extern int process_buffer2;			/* 2nd overlap buffer to be processed */
extern int len1;								/* Number of samples from 1st buffer */
extern int len2;								/* Number of samples from 2nd buffer */
extern int base_offset;					/* Start offset (in samples) in 1st buffer */

/* Graphics variables */

extern Rgb back, grat, graph, label, border;
extern Rgb text, high, low;
extern GC graph_gc, back_gc, text_gc, high_gc, grat_gc;
extern GC label_gc, border_gc, gback_gc;
extern GC low_gc, rain_gc;
extern Rgb dialog_back, dialog_fore, dialog_title;
extern Rgb dialog_dim, dialog_high;
extern GC dialog_back_gc, dialog_fore_gc, dialog_title_gc;
extern GC dialog_hl_black_gc, dialog_hl_white_gc;
extern GC dialog_dim_gc, dialog_high_gc;

extern int scrolled;						/* Flag for graph scrolled in freeze mode */
extern int first_peak;					/* Peak mode just set */
extern int first_bin;						/* First bin to be displayed */
extern int num_bins;						/* Number of bins displayed */
extern int zoom_factor;					/* X param factor */
extern Display *display;				/* Display device */
extern Window window;						/* Main window */
extern Pixmap shadow;						/* Main window shadow pixmap */
extern int depth;								/* Screen bit depth */
extern XSegment *graph_segs;		/* Array of segments for line drawing */
extern XRectangle *graph_rects;	/* Array of rectangles for drawing */
extern Atom wm_protocols;				/* For window manager */
extern Atom wm_delete_window;		/* WM_DELETE message */
extern unsigned long rainbow[];	/* Rainbow pixels array */
extern int rainbow_mode;				/* Colour rainbow_mode */
extern int graticule_mode;			/* Flag for graticule mode */

/* vi: set ts=2: */
