/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* fftw.c - FFTW routines for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
init_wisdom (void)
{
	FILE *fp;
	struct passwd *passwd_entry;
	char wisdom_file[80];
	int wisdom_done = 0;

/* Get home directory for 'wisdom' file ... */
	if ((passwd_entry = getpwuid (getuid ())) == NULL)
		error ("No such user.");

	strcpy (wisdom_file, passwd_entry->pw_dir);
	strcat (wisdom_file, MYWISDOM);

/* Try to open WISDOM file ... */
	if ((fp = fopen (wisdom_file, "r")) == NULL)
	{
		if ((fp = fopen (wisdom_file, "w")) == NULL)
			error ("Cannot create WISDOM file.");

		else
		/* Write a WISDOM file ... */
		{
			the_plan =
				fftw_plan_r2r_1d (MAX_LEN, fft_in, fft_out, MYDIREC, MYFLAGS);
			wisdom_done = 1;
			fftw_export_wisdom_to_file (fp);
		}
	}

	else
	/* WISDOM file exists ... */
		fftw_import_wisdom_from_file (fp);

	fclose (fp);

	if (wisdom_done == 0)
		the_plan = fftw_plan_r2r_1d (MAX_LEN, fft_in, fft_out, MYDIREC, MYFLAGS);

}																/* end init_wisdom */

void
get_power (void)
{
	int i;												/* General 'for' counter */

	double *pPowerval = power_val;	/* This power value array */
	double *re;										/* Real part of FFT out */
	double *im;										/* Imaginary part of FFT out */

	re = fft_out + 1;
	im = fft_out + MAX_LEN - 1;

	if (!averaging_mode)
	{
		*pPowerval++ = fft_out[0] * fft_out[0];

		for (i = MAX_LEN / 2 - 1; i; i--, re++, im--)
			*pPowerval++ = (*re) * (*re) + (*im) * (*im);

	}															/* end !averaging_mode */

	else
	/* averaging_mode */
	{
		double *pLastp = last_power;

		*pPowerval = ((*pLastp * averaging_beta) +
									(fft_out[0] * fft_out[0] * averaging_alpha));

		*pLastp++ = *pPowerval++;

		for (i = MAX_LEN / 2 - 1; i; i--, re++, im--)
		{
			*pPowerval = ((*pLastp * averaging_beta) +
										(((*re) * (*re) + (*im) * (*im)) * averaging_alpha));

			*pLastp++ = *pPowerval++;

		}														/* end for */
	}															/* end averaging_mode */
}																/* end get_power */

/* vi: set ts=2: */
