/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* global.c - globals for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

/* Buffer stuff (set later) */
int queue_buffer;								/* Number of next buffer to be queued */
int process_buffer1;						/* 1st buffer to be processed */
int process_buffer2;						/* 2nd buffer to be processed */
int len1;												/* Number of samples from 1st buffer */
int len2;												/* Number of samples from 2nd buffer */
int base_offset;								/* Start offset (in samples) in 1st buffer */
void *buffer[BUFFERS];					/* Buffers for gathering data */

char alsa_device[ALSA_LEN] = "default";	/* Space for the ALSA device string */

int first_peak = 0;							/* Peak mode just set */
double last_peak_f = RESETF;		/* Last peak frequency Hz (dummy initialize) */
double last_peak_dB = RESETDB;	/* Last peak power dB (dummy initialize) */

int *last_y;										/* Last y position */
double *power_val;							/* Display value buffer */
double *last_power;							/* Last power value for averaging mode */
double y_scale;									/* Scaling factor (set later) */
double y_offset;								/* Zero dB offset from log_top (set later) */

int scrolled = 1;								/* Flag for graph scrolled in freeze mode */
int first_bin = 0;							/* First bin to be displayed */
int num_bins = WIDE;						/* Number of bins displayed */
int zoom_factor;								/* X zoom factor (set later) */

fftw_plan the_plan;							/* Used for FFT plan */
double *fft_in;									/* Used for transforms */
double *fft_out;								/* Output of FFT transform */

volatile int quit_signal = 0;		/* Signal to quit received */
volatile int suspend_signal = 0;	/* Signal to suspend received */
volatile int resume_signal = 0;	/* Signal to resume received */

int help_on = 0;								/* Help window shown */
int about_on = 0;								/* About window shown */

char *mode_names[] = { "Mono", "Left", "Right" };
int allow_rates[] = { 8000, 11025, 16000, 22050, 32000, 44100, 48000 };

int stereo_channel = 0;					/* Which channel(s) to show */
int rate_index = 0;							/* Index in array of allowable sample rates */
int dc_offset = 0;							/* DC offset for samples */
int log_base = -60;							/* Base of log y scale (in dB) */
int log_top = -10;							/* Max. of log y scale (in dB) */

int rainbow_mode = 1;						/* 0 (normal) or 1 (rainbow) */
int graticule_mode = 0;					/* Flag for graticule mode */
int averaging_mode = 1;					/* Flag for averaging mode */
int averaging_factor = 50;			/* Extent of averaging (%) */
double averaging_alpha;					/* How much this frame (set later) */
double averaging_beta;					/* How much last frame (set later) */
int peak_mode = 0;							/* Peak mode off */
int freeze_mode = 0;						/* Flag indicating a paused display */
int overlap_mode = 1;						/* Flag for input block overlap mode */
int overlap_factor = 5;					/* Extent of overlap */

int sound_set = 0;							/* Flag for soundcard set */
double frame_rate;							/* Frames per second (set later) */
int frame_period;								/* Period in uS of frame rate (set later) */

int graph_set = 0;							/* Graphics set (one time) */
Display *display;								/* Display device */
Window window;									/* Main window */
Pixmap shadow;									/* Main window shadow pixmap */
int depth;											/* Screen bit depth */
Atom wm_protocols;							/* For window manager */
Atom wm_delete_window;					/* WM_DELETE message */

XSegment *graph_segs;						/* Array of segments for line drawing */
XRectangle *graph_rects;				/* Array of rectangles for drawing */

/* Main GCs (set later) */
GC graph_gc, back_gc, text_gc, high_gc;
GC grat_gc, label_gc, border_gc, gback_gc;
GC low_gc, rain_gc;

/* Main colour scheme */
Rgb back = { 15, 0, 0 };
Rgb high = { 63, 63, 0 };
Rgb graph = { 63, 40, 0 };
Rgb label = { 63, 20, 0 };
Rgb border = { 40, 15, 15 };
Rgb text = { 50, 50, 50 };
Rgb grat = { 25, 10, 10 };
Rgb low = { 36, 36, 36 };

int scr_list_len = 0;						/* How many menu windows */
int screen_list[MAX_DEPTH];			/* List of menu windows */

/* Dialog GCs (set later) */
GC dialog_back_gc, dialog_fore_gc, dialog_title_gc;
GC dialog_hl_black_gc, dialog_hl_white_gc, dialog_dim_gc, dialog_high_gc;

/* Dialog colour scheme */
Rgb dialog_back = { 52, 52, 52 };
Rgb dialog_fore = { 0, 0, 0 };
Rgb dialog_title = { 0, 32, 0 };
Rgb dialog_dim = { 32, 32, 32 };
Rgb dialog_high = { 0, 0, 40 };

/* vi: set ts=2: */
