/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* help.c - help, about & quit windows for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

static int help_created = 0;
static int about_created = 0;

static Window help_window, about_window;
static Pixmap help_shadow, about_shadow;

Help_item help_list[] = {
/* Column 1 */
	{"F10", 3, "Quit", 4},
	{"F1", 2, "Help", 4},
	{"F2", 2, "About", 5},
	{"Esc", 3, "Cancel", 6},
	{"d", 1, "Callibrate DC", 13},
	{"a", 1, "Toggle Averaging", 16},
	{"o", 1, "Toggle Overlap", 14},
	{"B", 1, "Averaging Up", 12},
	{"b", 1, "Averaging Down", 14},
	{"V", 1, "Overlap Up", 10},
	{"v", 1, "Overlap Down", 12},
	{"R", 1, "Sample Rate Up", 14},
	{"r", 1, "Sample Rate Down", 16},
	{"S", 1, "Stereo Up", 9},
	{"s", 1, "Stereo Down", 11},
/* Column 2 */
	{"p", 1, "Toggle Peak", 11},
	{"g", 1, "Toggle Graticule", 16},
	{"c", 1, "Toggle Rainbow", 14},
	{"Space", 5, "Toggle Freeze", 13},
	{"Pg Up", 5, "Y Zoom Out", 10},
	{"Pg Dn", 5, "Y Zoom In", 9},
	{"Up", 2, "Y Up", 4},
	{"Down", 4, "Y Down", 6},
	{"<", 1, "X Zoom In", 9},
	{">", 1, "X Zoom Out", 10},
	{"f", 1, "X Zoom Off", 10},
	{"Left", 4, "X Left", 6},
	{"Right", 5, "X Right", 7},
	{"Home", 4, "X Far Left", 10},
	{"End", 3, "X Far Right", 11}
};

void
show_help_window (int dummy)
{
	if (!help_created)
	/* Create the new window ... */
	{
		Help_item *hip;
		int count;
		XSizeHints *sizehints;

	/* Allocate space for hints */
		if ((sizehints = XAllocSizeHints ()) == 0)
			error ("Not enough memory for the size hints structure.");

	/* Create the window ... */
		help_window = XCreateSimpleWindow (display, window,
																			 HX, HY, HW, HH, 0, 0, 0);

	/* Create the shadow pixmap ... */
		help_shadow = XCreatePixmap (display, help_window, HW, HH, depth);

	/* Make shadow pixmap the window background pixmap ... */
		XSetWindowBackgroundPixmap (display, help_window, help_shadow);

	/* Don't allow resize ... and place according to program */
		sizehints->flags = PPosition | PSize | PMinSize | PMaxSize;
		sizehints->min_width = HW;
		sizehints->min_height = HH;
		sizehints->max_width = HW;
		sizehints->max_height = HH;

		XSetStandardProperties (display, help_window,
														NULL, NULL, 0, 0, 0, sizehints);

	/* Free up the hints space */
		XFree (sizehints);

	/* Help background ... */
		XFillRectangle (display, help_shadow, dialog_back_gc, 0, 0, HW, HH);

	/* First the help highlight outline */
		XDrawLine (display, help_shadow, dialog_hl_white_gc, 0, 0, HW - MHW, 0);
		XDrawLine (display, help_shadow, dialog_hl_white_gc, 0, MHW, 0, HH - MHW);

		XDrawLine (display, help_shadow, dialog_hl_black_gc,
							 HW - MHW, MHW, HW - MHW, HH - MHW);

		XDrawLine (display, help_shadow, dialog_hl_black_gc,
							 MHW, HH - MHW, HW - MHW, HH - MHW);

	/* Now the help title */
		XDrawString (display, help_shadow, dialog_title_gc, HTX, HTY, HT, HTL);

	/* A furrowed seperator */
		XDrawLine (display, help_shadow, dialog_hl_black_gc, MHW, ML1,
							 HW - MHW - 1, ML1);

		XDrawLine (display, help_shadow, dialog_hl_white_gc, MHW, ML2,
							 HW - MHW - 1, ML2);

	/* Now the text ... */
		hip = help_list;
		for (count = 0; count < HELP_TOTAL1; count++, hip++)
		/* Column 1 */
		{
			XDrawString (display, help_shadow, dialog_high_gc,
									 HC1K, count * MSP + M1, hip->key, hip->key_len);

			XDrawString (display, help_shadow, dialog_fore_gc,
									 HC1D, count * MSP + M1,
									 hip->description, hip->description_len);
		}

		for (count = 0; count < HELP_TOTAL2; count++, hip++)
		/* Column 2 */
		{
			XDrawString (display, help_shadow, dialog_high_gc,
									 HC2K, count * MSP + M1, hip->key, hip->key_len);

			XDrawString (display, help_shadow, dialog_fore_gc,
									 HC2D, count * MSP + M1,
									 hip->description, hip->description_len);
		}

	/* Look for button press events in the help window ... */
		XSelectInput (display, help_window, ButtonPressMask);
		help_created = 1;

	}															/* end if (!help_created) */

/* Fire up the help window */
	XMapRaised (display, help_window);
	help_on = 1;

}																/* end show_help_window */

void
close_help_window (void)
{
	XUnmapWindow (display, help_window);
	help_on = 0;

}																/* end close_help */

void
show_about_window (int dummy)
{
	if (!about_created)
	/* Create the new window ... */
	{
		XSizeHints *sizehints;

	/* Allocate space for hints */
		if ((sizehints = XAllocSizeHints ()) == 0)
			error ("Not enough memory for the size hints structure.");

	/* Create the window ... */
		about_window = XCreateSimpleWindow (display, window,
																				AX, AY, AW, AH, 0, 0, 0);

	/* Create the shadow pixmap ... */
		about_shadow = XCreatePixmap (display, about_window, AW, AH, depth);

	/* Make shadow pixmap the window background pixmap ... */
		XSetWindowBackgroundPixmap (display, about_window, about_shadow);

	/* Don't allow resize ... and place according to program */
		sizehints->flags = PPosition | PSize | PMinSize | PMaxSize;
		sizehints->min_width = AW;
		sizehints->min_height = AH;
		sizehints->max_width = AW;
		sizehints->max_height = AH;

		XSetStandardProperties (display, about_window,
														NULL, NULL, 0, 0, 0, sizehints);

	/* Free up the hints space */
		XFree (sizehints);

	/* About background ... */
		XFillRectangle (display, about_shadow, dialog_back_gc, 0, 0, AW, AH);

	/* First the help highlight outline */
		XDrawLine (display, about_shadow, dialog_hl_white_gc, 0, 0, AW - MHW, 0);
		XDrawLine (display, about_shadow, dialog_hl_white_gc, 0, MHW, 0,
							 AH - MHW);

		XDrawLine (display, about_shadow, dialog_hl_black_gc,
							 AW - MHW, MHW, AW - MHW, AH - MHW);

		XDrawLine (display, about_shadow, dialog_hl_black_gc,
							 MHW, AH - MHW, AW - MHW, AH - MHW);

	/* Now the help title */
		XDrawString (display, about_shadow, dialog_title_gc, ATX, ATY, AT, ATL);

	/* A furrowed seperator */
		XDrawLine (display, about_shadow, dialog_hl_black_gc, MHW, ML1,
							 AW - MHW - 1, ML1);

		XDrawLine (display, about_shadow, dialog_hl_white_gc, MHW, ML2,
							 AW - MHW - 1, ML2);

	/* About message */
		XDrawString (display, about_shadow, dialog_fore_gc, AMX, AMY, AM, AML);

	/* Look for button press events in the help window ... */
		XSelectInput (display, about_window, ButtonPressMask);
		about_created = 1;

	}															/* end if (!about_created) */

/* Fire up the about window */
	XMapRaised (display, about_window);
	about_on = 1;

}																/* end show_about_window */

void
close_about_window (void)
{
	XUnmapWindow (display, about_window);
	about_on = 0;

}																/* end close_about */

/* vi: set ts=2: */
