/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* inifile.c - read and write ini file for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

static char ini_file[80];				/* Filename for ini file */

void
parse_ini_file (void)
{
	FILE *fp;
	char buffer[80];
	char *p;
	struct passwd *passwd_entry;

/* Get home directory for 'ini' file ... */
	if ((passwd_entry = getpwuid (getuid ())) == NULL)
		error ("No such user.");

	strcpy (ini_file, passwd_entry->pw_dir);
	strcat (ini_file, DEF_INI_FILE);

	if ((fp = fopen (ini_file, "r")) != NULL)
	{
		while (!feof (fp))
		{
			fgets (buffer, sizeof (buffer), fp);

			if (*buffer == '#' || *buffer == '\n')
			/* Comment or blank line ... */
				continue;

			for (p = buffer; *p; p++)
				*p = toupper (*p);

			if (sscanf (buffer, "SAMPLE RATE: %d", &rate_index))
				continue;

			else if (sscanf (buffer, "STEREO CHANNEL: %d", &stereo_channel))
				continue;

			else if (sscanf (buffer, "DC OFFSET: %d", &dc_offset))
				continue;

			else if (sscanf (buffer, "RAINBOW MODE: %d", &rainbow_mode))
				continue;

			else if (sscanf (buffer, "AVERAGING MODE: %d", &averaging_mode))
				continue;

			else if (sscanf (buffer, "AVERAGING FACTOR: %d", &averaging_factor))
				continue;

			else if (sscanf (buffer, "PEAK MODE: %d", &peak_mode))
				continue;

			else if (sscanf (buffer, "OVERLAP MODE: %d", &overlap_mode))
				continue;

			else if (sscanf (buffer, "OVERLAP FACTOR: %d", &overlap_factor))
				continue;

			else if (sscanf (buffer, "GRATICULE MODE: %d", &graticule_mode))
				continue;

			else if (sscanf (buffer, "FREEZE MODE: %d", &freeze_mode))
				continue;

			else if (sscanf (buffer, "LOG BASE: %d", &log_base))
				continue;

			else if (sscanf (buffer, "LOG TOP: %d", &log_top))
				continue;

			else if (sscanf (buffer, "FIRST BIN: %d", &first_bin))
				continue;

			else if (sscanf (buffer, "NUM BINS: %d", &num_bins))
				continue;

			else if (sscanf (buffer, "BACKGROUND COLOUR: %d,%d,%d",
											 &back.red, &back.green, &back.blue))
				continue;

			else if (sscanf (buffer, "GRATICULE COLOUR: %d,%d,%d",
											 &grat.red, &grat.green, &grat.blue))
				continue;

			else if (sscanf (buffer, "HIGHLIGHT COLOUR: %d,%d,%d",
											 &high.red, &high.green, &high.blue))
				continue;

			else if (sscanf (buffer, "GRAPH COLOUR: %d,%d,%d",
											 &graph.red, &graph.green, &graph.blue))
				continue;

			else if (sscanf (buffer, "AXIS LABEL COLOUR: %d,%d,%d",
											 &label.red, &label.green, &label.blue))
				continue;

			else if (sscanf (buffer, "BORDER COLOUR: %d,%d,%d",
											 &border.red, &border.green, &border.blue))
				continue;

			else if (sscanf (buffer, "TEXT COLOUR: %d,%d,%d",
											 &text.red, &text.green, &text.blue))
				continue;

			else if (sscanf (buffer, "LOW COLOUR: %d,%d,%d",
											 &low.red, &low.green, &low.blue))
				continue;

			else if (sscanf (buffer, "DIALOG FORE COLOUR: %d,%d,%d",
											 &dialog_fore.red, &dialog_fore.green,
											 &dialog_fore.blue))
				continue;

			else if (sscanf (buffer, "DIALOG BACK COLOUR: %d,%d,%d",
											 &dialog_back.red, &dialog_back.green,
											 &dialog_back.blue))
				continue;

			else if (sscanf (buffer, "DIALOG TITLE COLOUR: %d,%d,%d",
											 &dialog_title.red, &dialog_title.green,
											 &dialog_title.blue))
				continue;

			else if (sscanf (buffer, "DIALOG DIM COLOUR: %d,%d,%d",
											 &dialog_dim.red, &dialog_dim.green, &dialog_dim.blue))
				continue;

			else if (sscanf (buffer, "DIALOG HIGH COLOUR: %d,%d,%d",
											 &dialog_high.red, &dialog_high.green,
											 &dialog_high.blue))
				continue;

		}														/* end while !feof ... */

		fclose (fp);

	}															/* end if fopen ... */
}																/* end parse_ini_file */

void
save_ini_file (void)
{
	FILE *fp;
	time_t the_time;

	if ((fp = fopen (ini_file, "w")) != NULL)
	{
		the_time = time (&the_time);

		fprintf (fp, "# .xspect.ini - created %s\n", ctime (&the_time));
		fputs ("# WARNING only edit this if you are sure ...\n\n", fp);

		fprintf (fp, "Sample rate: %d\n", rate_index);
		fprintf (fp, "Stereo channel: %d\n", stereo_channel);
		fprintf (fp, "DC offset: %d\n", dc_offset);
		fprintf (fp, "Rainbow mode: %d\n", rainbow_mode);
		fprintf (fp, "Averaging mode: %d\n", averaging_mode);
		fprintf (fp, "Averaging factor: %d\n", averaging_factor);
		fprintf (fp, "Peak mode: %d\n", peak_mode);
		fprintf (fp, "Overlap mode: %d\n", overlap_mode);
		fprintf (fp, "Overlap factor: %d\n", overlap_factor);
		fprintf (fp, "Graticule mode: %d\n", graticule_mode);
		fprintf (fp, "Freeze mode: %d\n", freeze_mode);

		fputs ("\n# These are not on the command line ...\n\n", fp);

		fprintf (fp, "Log base: %d\n", log_base);
		fprintf (fp, "Log top: %d\n", log_top);
		fprintf (fp, "First bin: %d\n", first_bin);
		fprintf (fp, "Num bins: %d\n", num_bins);

		fputs ("\n# The only way to change the colours ...\n\n", fp);

		fprintf (fp, "Background colour: %d,%d,%d\n",
						 back.red, back.green, back.blue);

		fprintf (fp, "Graticule colour: %d,%d,%d\n",
						 grat.red, grat.green, grat.blue);

		fprintf (fp, "Highlight colour: %d,%d,%d\n",
						 high.red, high.green, high.blue);

		fprintf (fp, "Graph colour: %d,%d,%d\n",
						 graph.red, graph.green, graph.blue);

		fprintf (fp, "Axis label colour: %d,%d,%d\n",
						 label.red, label.green, label.blue);

		fprintf (fp, "Border colour: %d,%d,%d\n",
						 border.red, border.green, border.blue);

		fprintf (fp, "Text colour: %d,%d,%d\n", text.red, text.green, text.blue);
		fprintf (fp, "Low colour: %d,%d,%d\n", low.red, low.green, low.blue);

		fprintf (fp, "Dialog fore colour: %d,%d,%d\n",
						 dialog_fore.red, dialog_fore.green, dialog_fore.blue);

		fprintf (fp, "Dialog back colour: %d,%d,%d\n",
						 dialog_back.red, dialog_back.green, dialog_back.blue);

		fprintf (fp, "Dialog title colour: %d,%d,%d\n",
						 dialog_title.red, dialog_title.green, dialog_title.blue);

		fprintf (fp, "Dialog dim colour: %d,%d,%d\n",
						 dialog_dim.red, dialog_dim.green, dialog_dim.blue);

		fprintf (fp, "Dialog high colour: %d,%d,%d\n",
						 dialog_high.red, dialog_high.green, dialog_high.blue);

		fclose (fp);

	}															/* end OK file */
}																/* end save_ini_file */

/* vi: set ts=2: */
