/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* menudef.c - menu definitions for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

static Menu_item top_list[] = {
	{"Sampling", 8, TYPE_SUB, NULL, DUMMY, freeze_on, 1},
	{"Graph", 5, TYPE_SUB, NULL, DUMMY, NULL, 2},
	{"Toggle", 6, TYPE_SUB, NULL, DUMMY, NULL, 3},
	{"Help", 4, TYPE_SUB, NULL, DUMMY, NULL, 4},
	{"Quit", 4, TYPE_SUB, NULL, DUMMY, NULL, 5}
};

static Menu_item sample_list[] = {
	{"Rate", 4, TYPE_SUB, NULL, DUMMY, NULL, 6},
	{"Overlap", 7, TYPE_SUB, NULL, DUMMY, overlap_off, 7},
	{"Stereo", 6, TYPE_SUB, NULL, DUMMY, freeze_on, 8},
	{"Averaging", 9, TYPE_SUB, NULL, DUMMY, averaging_off, 9},
	{"DC Offset", 9, TYPE_CALL, do_callibrate_dc, DUMMY, NULL, DUMMY}
};

static Menu_item graph_list[] = {
	{"X Axis", 6, TYPE_SUB, NULL, DUMMY, NULL, 10},
	{"Y Axis", 6, TYPE_SUB, NULL, DUMMY, NULL, 11}
};

static Menu_item toggle_list[] = {
	{"Freeze", 6, TYPE_CALL, do_toggle_freeze, DUMMY, NULL, DUMMY},
	{"Overlap", 7, TYPE_CALL, do_toggle_overlap, DUMMY, freeze_on, DUMMY},
	{"Averaging", 9, TYPE_CALL, do_toggle_averaging, DUMMY, freeze_on, DUMMY},
	{"Peak", 4, TYPE_CALL, do_toggle_peak, DUMMY, NULL, DUMMY},
	{"Graticule", 9, TYPE_CALL, do_toggle_graticule, DUMMY, NULL, DUMMY},
	{"Rainbow", 7, TYPE_CALL, do_toggle_rainbow, DUMMY, NULL, DUMMY}
};

static Menu_item help_list[] = {
	{"Keys", 4, TYPE_DIAL, show_help_window, DUMMY, NULL, DUMMY},
	{"About", 5, TYPE_DIAL, show_about_window, DUMMY, NULL, DUMMY}
};

static Menu_item quit_list[] = {
	{"Cancel", 6, TYPE_NULL, NULL, DUMMY, NULL, DUMMY},
	{"Quit", 4, TYPE_CALL, do_quit, DUMMY, NULL, DUMMY}
};

static Menu_item rate_list[] = {
	{"8000 ", 5, TYPE_CALL, do_set_rate, 0, rate_is, DUMMY},
	{"11025", 5, TYPE_CALL, do_set_rate, 1, rate_is, DUMMY},
	{"16000", 5, TYPE_CALL, do_set_rate, 2, rate_is, DUMMY},
	{"22050", 5, TYPE_CALL, do_set_rate, 3, rate_is, DUMMY},
	{"32000", 5, TYPE_CALL, do_set_rate, 4, rate_is, DUMMY},
	{"44100", 5, TYPE_CALL, do_set_rate, 5, rate_is, DUMMY},
	{"48000", 5, TYPE_CALL, do_set_rate, 6, rate_is, DUMMY}
};

static Menu_item overlap_list[] = {
	{"2", 1, TYPE_CALL, do_set_overlap, 2, overlap_is, DUMMY},
	{"3", 1, TYPE_CALL, do_set_overlap, 3, overlap_is, DUMMY},
	{"4", 1, TYPE_CALL, do_set_overlap, 4, overlap_is, DUMMY},
	{"5", 1, TYPE_CALL, do_set_overlap, 5, overlap_is, DUMMY},
	{"6", 1, TYPE_CALL, do_set_overlap, 6, overlap_is, DUMMY},
	{"7", 1, TYPE_CALL, do_set_overlap, 7, overlap_is, DUMMY},
	{"8", 1, TYPE_CALL, do_set_overlap, 8, overlap_is, DUMMY},
	{"9", 1, TYPE_CALL, do_set_overlap, 9, overlap_is, DUMMY}
};

static Menu_item stereo_list[] = {
	{"Mono", 4, TYPE_CALL, do_set_stereo, 0, channel_is, DUMMY},
	{"Left", 4, TYPE_CALL, do_set_stereo, 1, channel_is, DUMMY},
	{"Right", 5, TYPE_CALL, do_set_stereo, 2, channel_is, DUMMY}
};

static Menu_item x_list[] = {
	{"Zoom", 4, TYPE_SUB, NULL, DUMMY, NULL, 12},
	{"Left", 4, TYPE_CALL, do_horizontal_left, DUMMY, x_zoom_start, DUMMY},
	{"Right", 5, TYPE_CALL, do_horizontal_right, DUMMY, x_zoom_end, DUMMY},
	{"Start", 5, TYPE_CALL, do_horizontal_start, DUMMY, x_zoom_start, DUMMY},
	{"End", 3, TYPE_CALL, do_horizontal_end, DUMMY, x_zoom_end, DUMMY}
};

static Menu_item y_list[] = {
	{"Zoom", 4, TYPE_SUB, NULL, DUMMY, NULL, 13},
	{"Up", 2, TYPE_CALL, do_vertical_up, DUMMY, y_zoom_top, DUMMY},
	{"Down", 4, TYPE_CALL, do_vertical_down, DUMMY, y_zoom_bottom, DUMMY},
	{"Top", 3, TYPE_CALL, do_vertical_top, DUMMY, y_zoom_top, DUMMY},
	{"Bottom", 6, TYPE_CALL, do_vertical_bottom, DUMMY, y_zoom_bottom, DUMMY}
};

static Menu_item averaging_list[] = {
	{"10", 2, TYPE_CALL, do_set_averaging, 10, averaging_is, DUMMY},
	{"20", 2, TYPE_CALL, do_set_averaging, 20, averaging_is, DUMMY},
	{"30", 2, TYPE_CALL, do_set_averaging, 30, averaging_is, DUMMY},
	{"40", 2, TYPE_CALL, do_set_averaging, 40, averaging_is, DUMMY},
	{"50", 2, TYPE_CALL, do_set_averaging, 50, averaging_is, DUMMY},
	{"60", 2, TYPE_CALL, do_set_averaging, 60, averaging_is, DUMMY},
	{"70", 2, TYPE_CALL, do_set_averaging, 70, averaging_is, DUMMY},
	{"80", 2, TYPE_CALL, do_set_averaging, 80, averaging_is, DUMMY},
	{"90", 2, TYPE_CALL, do_set_averaging, 90, averaging_is, DUMMY}
};

static Menu_item x_zoom_list[] = {
	{"Off", 3, TYPE_CALL, do_horizontal_zoom, 1, x_zoom_is, DUMMY},
	{"2", 1, TYPE_CALL, do_horizontal_zoom, 2, x_zoom_is, DUMMY},
	{"4", 1, TYPE_CALL, do_horizontal_zoom, 4, x_zoom_is, DUMMY},
	{"8", 1, TYPE_CALL, do_horizontal_zoom, 8, x_zoom_is, DUMMY},
	{"16", 2, TYPE_CALL, do_horizontal_zoom, 16, x_zoom_is, DUMMY},
	{"32", 2, TYPE_CALL, do_horizontal_zoom, 32, x_zoom_is, DUMMY}
};

static Menu_item y_zoom_list[] = {
	{"Off", 3, TYPE_CALL, do_vertical_full, DUMMY, y_zoom_off, DUMMY},
	{"In", 2, TYPE_CALL, do_vertical_less, DUMMY, y_zoom_max, DUMMY},
	{"Out", 3, TYPE_CALL, do_vertical_more, DUMMY, y_zoom_off, DUMMY}
};

Menu menus[MENU_LENGTH] = {
	{"xspect - DSP", 12, top_list, 5, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Sampling", 8, sample_list, 5, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Graph", 5, graph_list, 2, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Toggle", 6, toggle_list, 6, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Help", 4, help_list, 2, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Quit", 4, quit_list, 2, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Rate", 4, rate_list, 7, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Overlap", 7, overlap_list, 8, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Stereo", 6, stereo_list, 3, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Averaging", 9, averaging_list, 9, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"X Axis", 6, x_list, 5, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Y Axis", 6, y_list, 5, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Zoom", 4, x_zoom_list, 6, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	{"Zoom", 4, y_zoom_list, 3, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

/* vi: set ts=2: */
