/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* motion.c - scrolling the `xspect' graph by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
do_vertical_down (int dummy)
{
	log_base -= LOGMIN;
	log_top -= LOGMIN;
	setup_logscales ();
	power_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_down */

void
do_vertical_up (int dummy)
{
	log_top += LOGMIN;
	log_base += LOGMIN;
	setup_logscales ();
	power_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_up */

void
do_vertical_more (int dummy)
{
	if (graticule_mode)
		graticule_horizontal ();

	if (log_top != LOGTOPMAX)
		log_top += LOGMIN;

	else if (log_base != LOGBASEMIN)
		log_base -= LOGMIN;

	setup_logscales ();
	power_scale ();

	if (graticule_mode)
		graticule_horizontal ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_more */

void
do_vertical_less (int dummy)
{
	if (graticule_mode)
		graticule_horizontal ();

	log_top -= LOGMIN;
	setup_logscales ();
	power_scale ();

	if (graticule_mode)
		graticule_horizontal ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_less */

void
do_vertical_top (int dummy)
{
	int diff = log_top - log_base;

	log_top = LOGTOPMAX;
	log_base = LOGTOPMAX - diff;
	setup_logscales ();
	power_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_top */

void
do_vertical_bottom (int dummy)
{
	int diff = log_top - log_base;

	log_base = LOGBASEMIN;
	log_top = log_base + diff;
	setup_logscales ();
	power_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_bottom */

void
do_vertical_full (int dummy)
{
	if (graticule_mode)
		graticule_horizontal ();

	log_top = LOGTOPMAX;
	log_base = LOGBASEMIN;
	setup_logscales ();
	power_scale ();

	if (graticule_mode)
		graticule_horizontal ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_vertical_full */

void
do_horizontal_start (int dummy)
{
	first_bin = BINMIN;

	if (rainbow_mode)
	{
		reset_last_y ();
		graph_background ();

		if (graticule_mode)
		{
			graticule_horizontal ();
			graticule_vertical ();
		}
	}

	frequency_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_start */

void
do_horizontal_end (int dummy)
{
	first_bin = WIDE - num_bins;

	if (rainbow_mode)
	{
		reset_last_y ();
		graph_background ();

		if (graticule_mode)
		{
			graticule_horizontal ();
			graticule_vertical ();
		}
	}

	frequency_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_end */

void
do_horizontal_full (int dummy)
{
	first_bin = BINMIN;
	num_bins = WIDE;
	zoom_factor = 1;
	reset_last_y ();
	graph_background ();

	if (graticule_mode)
	{
		graticule_horizontal ();
		graticule_vertical ();
	}

	frequency_scale ();
	update_zoom ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_full */

void
do_horizontal_more (int dummy)
{
	first_bin -= num_bins / (XZOOM);
	num_bins *= XZOOM;

	if (first_bin < BINMIN)
		first_bin = BINMIN;

	else if (first_bin + num_bins >= WIDE)
		first_bin = WIDE - num_bins;

	zoom_factor /= XZOOM;
	reset_last_y ();
	graph_background ();

	if (graticule_mode)
	{
		graticule_horizontal ();
		graticule_vertical ();
	}

	frequency_scale ();
	update_zoom ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_more */

void
do_horizontal_less (int dummy)
{
	num_bins /= XZOOM;
	first_bin += num_bins / (XZOOM);
	zoom_factor *= XZOOM;
	reset_last_y ();
	graph_background ();

	if (graticule_mode)
	{
		graticule_horizontal ();
		graticule_vertical ();
	}

	frequency_scale ();
	update_zoom ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_less */

void
do_horizontal_left (int dummy)
{
	first_bin -= num_bins / XOVER;

	if (rainbow_mode)
	{
		reset_last_y ();
		graph_background ();

		if (graticule_mode)
		{
			graticule_horizontal ();
			graticule_vertical ();
		}
	}

	frequency_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_left */

void
do_horizontal_right (int dummy)
{
	first_bin += num_bins / XOVER;

	if (rainbow_mode)
	{
		reset_last_y ();
		graph_background ();

		if (graticule_mode)
		{
			graticule_horizontal ();
			graticule_vertical ();
		}
	}

	frequency_scale ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_right */

void
do_horizontal_zoom (int param)
{
	while (param > zoom_factor)
	{
		num_bins /= XZOOM;
		first_bin += num_bins / (XZOOM);
		zoom_factor *= XZOOM;
	}

	while (param < zoom_factor)
	{
		first_bin -= num_bins / (XZOOM);
		num_bins *= XZOOM;

		if (first_bin < BINMIN)
			first_bin = BINMIN;

		else if (first_bin + num_bins >= WIDE)
			first_bin = WIDE - num_bins;

		zoom_factor /= XZOOM;
	}

	reset_last_y ();
	graph_background ();

	if (graticule_mode)
	{
		graticule_horizontal ();
		graticule_vertical ();
	}

	frequency_scale ();
	update_zoom ();

	if (freeze_mode)
		scrolled = 1;

}																/* end do_horizontal_zoom */

/* vi: set ts=2: */
