/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
   *  palette.c
   *
   *  Get a TrueColor rainbow palette for xspect
   *
   *  Copyright (C) 2007 Gordon Miller
   *
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#define  WNAME					"Rainbow Palette - Click to exit"

#define  WIDE           (512)
#define  LEVELS					(64)
#define  SLICES					(LEVELS-1)
#define  PSIZE					(WIDE*SLICES)
#define  LNUM           (WIDE/5)
#define  CMAX           (65535)
#define  CGAP           ((CMAX+1)/LNUM)

int
main (void)
{
	int screen;
	int n, inner, r = CMAX, g = 0, b = 0;
	int x, y;
	unsigned long colrainb[PSIZE];
	unsigned long *ulp = colrainb, *iulp;

	Display *display;
	Colormap colourmap;
	XColor x_colour;
	Window window;
	XSizeHints *sizehints;
	XSetWindowAttributes winatts;
	Atom wm_protocols;
	Atom wm_delete_window;

	XGCValues values;
	GC rain_gc;
	XEvent event;

/* Try to open display */

	if ((display = XOpenDisplay (NULL)) == 0)
	{
		puts ("\nCannot open X connection.\n");
		exit (EXIT_FAILURE);
	}

	screen = DefaultScreen (display);

	if (DefaultVisual (display, screen)->class != TrueColor)
	{
		puts ("\nCan only run in TrueColor.\n");
		XCloseDisplay (display);
		exit (EXIT_FAILURE);
	}

	colourmap = DefaultColormap (display, screen);

/* Set up a blank window */
	window = XCreateSimpleWindow (display, DefaultRootWindow (display),
																0, 0, WIDE, SLICES, 0, 0, 0);

/* Allocate space for hints */
	sizehints = XAllocSizeHints ();

/* Don't allow resize ... and place according to program */
	sizehints->flags = PPosition | PSize | PMinSize | PMaxSize;

	sizehints->min_width = WIDE;
	sizehints->min_height = SLICES;
	sizehints->max_width = WIDE;
	sizehints->max_height = SLICES;

/* Set window name, icon name & sizehints */
	XSetStandardProperties (display, window, WNAME, WNAME, 0, 0, 0, sizehints);

/* Free up the hints space */
	XFree (sizehints);

/* Setup for ICCCM delete window. */
	wm_protocols = XInternAtom (display, "WM_PROTOCOLS", False);
	wm_delete_window = XInternAtom (display, "WM_DELETE_WINDOW", False);
	XSetWMProtocols (display, window, &wm_delete_window, 1);

/* Set up for backing store */
	winatts.backing_store = Always;
	XChangeWindowAttributes (display, window, CWBackingStore, &winatts);

/* Set the GCs for drawing */

	rain_gc = XCreateGC (display, window, 0, &values);

/* Catch key & button events only */
	XSelectInput (display, window, KeyPressMask | ButtonPressMask);

/* Fill rainbow palette */

	for (n = LNUM; n; n--, g += CGAP, ulp++)
		for (iulp = ulp, inner = SLICES + 1; inner > 1; inner--, iulp += WIDE)
		{
			x_colour.red = (r * inner) / LEVELS;
			x_colour.green = (g * inner) / LEVELS;
			x_colour.blue = (b * inner) / LEVELS;
			XAllocColor (display, colourmap, &x_colour);
			*iulp = x_colour.pixel;
		}

	for (n = LNUM; n; n--, r -= CGAP, ulp++)
		for (iulp = ulp, inner = SLICES + 1; inner > 1; inner--, iulp += WIDE)
		{
			x_colour.red = (r * inner) / LEVELS;
			x_colour.green = (g * inner) / LEVELS;
			x_colour.blue = (b * inner) / LEVELS;
			XAllocColor (display, colourmap, &x_colour);
			*iulp = x_colour.pixel;
		}

	for (n = LNUM; n; n--, b += CGAP, ulp++)
		for (iulp = ulp, inner = SLICES + 1; inner > 1; inner--, iulp += WIDE)
		{
			x_colour.red = (r * inner) / LEVELS;
			x_colour.green = (g * inner) / LEVELS;
			x_colour.blue = (b * inner) / LEVELS;
			XAllocColor (display, colourmap, &x_colour);
			*iulp = x_colour.pixel;
		}

	for (n = LNUM; n; n--, g -= CGAP, ulp++)
		for (iulp = ulp, inner = SLICES + 1; inner > 1; inner--, iulp += WIDE)
		{
			x_colour.red = (r * inner) / LEVELS;
			x_colour.green = (g * inner) / LEVELS;
			x_colour.blue = (b * inner) / LEVELS;
			XAllocColor (display, colourmap, &x_colour);
			*iulp = x_colour.pixel;
		}

	for (n = LNUM; n; n--, r += CGAP, ulp++)
		for (iulp = ulp, inner = SLICES + 1; inner > 1; inner--, iulp += WIDE)
		{
			x_colour.red = (r * inner) / LEVELS;
			x_colour.green = (g * inner) / LEVELS;
			x_colour.blue = (b * inner) / LEVELS;
			XAllocColor (display, colourmap, &x_colour);
			*iulp = x_colour.pixel;
		}

	for (n = 5 * LNUM; n < WIDE; n++, ulp++)
		for (iulp = ulp, inner = SLICES + 1; inner > 1; inner--, iulp += WIDE)
		{
			x_colour.red = (r * inner) / LEVELS;
			x_colour.green = (g * inner) / LEVELS;
			x_colour.blue = (b * inner) / LEVELS;
			XAllocColor (display, colourmap, &x_colour);
			*iulp = x_colour.pixel;
		}

/* Fire up the window ... */
	XMapRaised (display, window);
	XSync (display, False);

/* Fill in the dots ... */
	for (ulp = colrainb, x = 0; x < WIDE; x++, ulp++)
		for (iulp = ulp, y = SLICES - 1; y; y--, iulp += WIDE)
		{
			XSetForeground (display, rain_gc, *iulp);
			XDrawPoint (display, window, rain_gc, x, y);
		}

/* Wait for key or button ... */
	XNextEvent (display, &event);
	XCloseDisplay (display);

/* Print rainbow file */

	puts
		("/***************************************************************************");
	puts
		(" *   Copyright (C) 2007 by Gordon Miller                                   *");
	puts
		(" *                                                                         *");
	puts
		(" *   This program is free software; you can redistribute it and/or modify  *");
	puts
		(" *   it under the terms of the GNU General Public License as published by  *");
	puts
		(" *   the Free Software Foundation; either version 2 of the License, or     *");
	puts
		(" *   (at your option) any later version.                                   *");
	puts
		(" *                                                                         *");
	puts
		(" *   This program is distributed in the hope that it will be useful,       *");
	puts
		(" *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *");
	puts
		(" *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *");
	puts
		(" *   GNU General Public License for more details.                          *");
	puts
		(" *                                                                         *");
	puts
		(" *   You should have received a copy of the GNU General Public License     *");
	puts
		(" *   along with this program; if not, write to the                         *");
	puts
		(" *   Free Software Foundation, Inc.,                                       *");
	puts
		(" *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *");
	puts
		(" ***************************************************************************/\n");

	puts ("/* rainbow.c - A TrueColor palette (unsigned longs in HEX) */");
	puts ("/* Automatically generated for `xspect' by Gordon Miller 2007 */");
	puts ("/* WARNING do not edit */\n");
	puts ("unsigned long rainbow[] = {\n");

	for (n = 0, ulp = colrainb; n < PSIZE - 1; n++)
	{
		if (n % 5 == 0)
			printf ("\t");

		printf ("0x%08lx, ", *ulp++);

		if (n % 5 == 4)
			printf ("\n");

	}

	printf ("0x%08lx\n};\n", *ulp);

	exit (EXIT_SUCCESS);

}																/* end main */
