/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* peak.c - draw `xspect' peak indicator by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
draw_peak (void)
{
	double peak_power;						/* Peak power found */
	int peak_index;								/* Bin number of the peak power */
	double peak_f;								/* Peak freq in Hz */
	double peak_dB;								/* Peak power in dB */
	char str[8];
	double *pv = power_val + first_bin;
	int count;

	peak_power = 0.0;
	peak_index = 0;

	for (count = 0; count < num_bins; count++, pv++)
		if (*pv > peak_power)
		{
			peak_index = count;
			peak_power = *pv;
		}

	peak_index += first_bin;
	peak_f = (double) allow_rates[rate_index] * peak_index / MAX_LEN;
	peak_dB = (peak_power == 0.0) ? LOWPEAK : 10.0 * log10 (peak_power);

	if (peak_dB < LOWPEAK)
		peak_dB = LOWPEAK;

	if (peak_dB != last_peak_dB)
	/* Write new power */
	{
		last_peak_dB = peak_dB;
		sprintf (str, "%6.1f", peak_dB);

		XDrawImageString (display, shadow, high_gc,
											PKX + 5 * FONT_WIDTH, PKY, str, 6);

	}															/* end new power */

	if (peak_f != last_peak_f)
	/* Write new freq */
	{
		last_peak_f = peak_f;
		sprintf (str, "%7.1f", peak_f);

		XDrawImageString (display, shadow, high_gc,
											PKX + 15 * FONT_WIDTH, PKY, str, 7);

	}															/* end new freq */
}																/* end draw_peak */

/* vi: set ts=2: */
