/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* scales.c - setup X and Y scales for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
setup_logscales (void)
{
/* Amplitude values range from 1 to 0 */
/* Logs (base 10) range from 0 to -infinity */

/* The number of pixels per Bel ... */
	y_scale = -10.0 * HIGH / (log_base - log_top);

/* The number of pixels from "0dB" to WINDOW_TOP ... */
	y_offset = y_scale * log_top / -10.0;

}																/* end setup_logscales */

void
frequency_scale (void)
{
	int x = WINDOW_LEFT - FONT_WIDTH / 2;
	double freq = (double) allow_rates[rate_index] * first_bin / MAX_LEN;
	double step = (double) allow_rates[rate_index] / (XSTEP * WIDE / num_bins);

	for (; x <= WINDOW_RIGHT; x += XSTEP, freq += step)
	{
		int y = WINDOW_BOTTOM + 19;
		char text[6];
		char *sp = text;

		sprintf (text, "%-5.0f", freq);

		while (*sp)
		{
			XDrawImageString (display, shadow, label_gc, x, y, sp++, 1);
			y += FONT_HEIGHT;

		}														/* end while */
	}															/* end for */
}																/* end frequency_scale */

void
power_scale (void)
{
	int i;
	int yval = log_top;

/* Clear previous scale ... */
	XFillRectangle (display, shadow, back_gc,
									0, WINDOW_TOP - 8, WINDOW_LEFT - 1, HIGH + 14);

	XFillRectangle (display, shadow, back_gc,
									WINDOW_RIGHT + 2, WINDOW_TOP - 8, W - WINDOW_RIGHT - 4,
									HIGH + 14);

/* Tick marks ... */
	for (i = 1; i < (log_top - log_base) / 10; i++)
	{
		int y = WINDOW_TOP + rint (i * y_scale) - 1;

		XDrawLine (display, shadow, border_gc,
							 WINDOW_LEFT - 2, y, WINDOW_LEFT - 6, y);

		XDrawLine (display, shadow, border_gc,
							 WINDOW_RIGHT + 2, y, WINDOW_RIGHT + 6, y);

	}															/* end for */

/* Numbers ... */
	for (i = 0; i <= (log_top - log_base) / 10; i++, yval -= 10)
	{
		int y = WINDOW_TOP + rint (i * y_scale) - 1;
		int len;
		char text[5];

		sprintf (text, "%d", yval);
		len = strlen (text);

		XDrawString (display, shadow, label_gc,
								 WINDOW_LEFT - 9 - len * FONT_WIDTH,
								 y + FONT_HEIGHT / 2 - 1, text, len);

		XDrawString (display, shadow, label_gc,
								 WINDOW_RIGHT + 9, y + FONT_HEIGHT / 2 - 1, text, len);

	}															/* end for */
}																/* end power_scale */

/* vi: set ts=2: */
