/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* setup.c - setup for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
initialize (int argc, char *argv[])
{
	main_signals ();
	parse_ini_file ();
	parse_command_line (argc, argv);
	check_parameters ();
	setup_buffers ();
	reset_last_y ();
	reset_zoom ();
	init_frame_rate ();

/* Initialize the transforms ... */
	init_wisdom ();

	if (averaging_mode)
		reset_averaging ();

	if (overlap_mode)
		reset_overlap ();

/* Start recording ... */
	if (!freeze_mode)
		init_record ();

/* Start graphics operations ... */
	setup_graphics ();
	setup_palette ();
	erase_canvas ();
	graph_border ();
	graph_background ();

	frequency_scale ();
	setup_logscales ();
	power_scale ();
	init_header ();

	if (graticule_mode)
	{
		graticule_horizontal ();
		graticule_vertical ();
	}

	if (peak_mode)
		peak_string ();

/* Start dialog thread(s) ... */
	start_threads ();

}																/* end initialize */

void
setup_buffers (void)
{
	int i;

	for (i = 0; i < BUFFERS; i++)
		if ((buffer[i] = malloc (MAX_LEN * BPSSTEREO)) == NULL)
			error ("Unable to allocate enough memory for the buffers.");

	if ((fft_in = malloc (MAX_LEN * sizeof (double))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

	if ((fft_out = malloc (MAX_LEN * sizeof (double))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

	if ((power_val = malloc (MAX_LEN / 2 * sizeof (double))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

	if ((last_power = malloc (MAX_LEN / 2 * sizeof (double))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

	if ((last_y = malloc (WIDE * sizeof (int))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

	if ((graph_segs = malloc (WIDE * sizeof (XSegment))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

	if ((graph_rects = malloc (WIDE / 2 * sizeof (XRectangle))) == NULL)
		error ("Unable to allocate enough memory for the buffers.");

}																/* end setup_buffers */

void
error (char *str)
{
	if (sound_set)
		reset_soundcard ();

	if (graph_set)
		cleanup_graphics ();

	fprintf (stderr, "\n*** " PROGNAME " : %s ***\n\n", str);
	exit (EXIT_FAILURE);

}																/* end error */

/* vi: set ts=2: */
