/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* signal.c - signal handling  for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
main_signals (void)
/* Main thread before threading ... */
{
	sigset_t myset;
	struct sigaction term_action;
	struct sigaction suspend_action;
	struct sigaction resume_action;

/* Block termination and suspend signals ... */
	sigemptyset (&myset);
	sigaddset (&myset, SIGTERM);
	sigaddset (&myset, SIGINT);
	sigaddset (&myset, SIGQUIT);
	sigaddset (&myset, SIGHUP);
	sigaddset (&myset, SIGTSTP);
	sigaddset (&myset, SIGCONT);
	sigprocmask (SIG_BLOCK, &myset, NULL);

/* Set termination signals handler ... for dialog thread */
	term_action.sa_flags = 0;
	term_action.sa_handler = termination_handler;
	sigaction (SIGTERM, &term_action, NULL);
	sigaction (SIGINT, &term_action, NULL);
	sigaction (SIGQUIT, &term_action, NULL);
	sigaction (SIGHUP, &term_action, NULL);

/* Set suspend signal handler ... for dialog thread */
	suspend_action.sa_flags = 0;
	suspend_action.sa_handler = suspend_handler;
	sigaction (SIGTSTP, &suspend_action, NULL);

/* Set resume signal handler ... for dialog thread */
	resume_action.sa_flags = 0;
	resume_action.sa_handler = resume_handler;
	sigaction (SIGCONT, &resume_action, NULL);

}																/* end catch_signals */

void
dialog_signals (void)
/* Signals caught by dialog thread ... */
{
	sigset_t myset;

/* Unblock termination and suspend signals ... */
	sigemptyset (&myset);
	sigaddset (&myset, SIGTERM);
	sigaddset (&myset, SIGINT);
	sigaddset (&myset, SIGQUIT);
	sigaddset (&myset, SIGHUP);
	sigaddset (&myset, SIGTSTP);
	sigaddset (&myset, SIGCONT);
	pthread_sigmask (SIG_UNBLOCK, &myset, NULL);

}																/* end dialog_signals */

void
termination_handler (int signum)
/* Only called in dialog thread ... */
{
	quit_signal = 1;

}																/* end termination_handler */

void
suspend_handler (int signum)
/* Only called in dialog thread ... */
{
	suspend_signal = 1;

}																/* end suspend_handler */

void
resume_handler (int signum)
/* Only called in dialog thread ... */
{
	resume_signal = 1;

}																/* end resume_handler */

/* vi: set ts=2: */
