/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* coeffs.c - Make the Blackman data windowing tables > stdout */
/* Adjusted for windowing power loss, data scaling and FFTW normalization */
/* comes out in C format ... pipe to `window.c' for `xspect' ... */

/* Make sure that sizeof (unsigned long long int) == sizeof (double) ... */

#include "../xspect.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define BLACK1		(0.42)
#define BLACK2		(0.08)
#define BRAISE		(0.50)

#define DSCALE		(32768.0)

#define PERLINE		(3)
#define GFORMAT		"0x%016llxULL"	/* 8 byte HEX format */

int
main (void)
{
	int i;
	double wind[MAX_LEN];
	unsigned long long int *uip = (unsigned long long int *) wind;
	double *fp1 = wind, *fp2 = wind + MAX_LEN - 1;
	int count = (MAX_LEN / PERLINE) * PERLINE, stop;
	double sum = 0.0;
	double again;

/* Make sure that the word sizes are OK ... */
	if (sizeof (double) != sizeof (unsigned long long int))
	{
		puts ("ERROR sizeof (double) != sizeof (unsigned long long int)");
		exit (EXIT_FAILURE);
	}

/* First the prologue ... */

	puts
		("/***************************************************************************");
	puts
		(" *   Copyright (C) 2007 by Gordon Miller                                   *");
	puts
		(" *                                                                         *");
	puts
		(" *   This program is free software; you can redistribute it and/or modify  *");
	puts
		(" *   it under the terms of the GNU General Public License as published by  *");
	puts
		(" *   the Free Software Foundation; either version 2 of the License, or     *");
	puts
		(" *   (at your option) any later version.                                   *");
	puts
		(" *                                                                         *");
	puts
		(" *   This program is distributed in the hope that it will be useful,       *");
	puts
		(" *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *");
	puts
		(" *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *");
	puts
		(" *   GNU General Public License for more details.                          *");
	puts
		(" *                                                                         *");
	puts
		(" *   You should have received a copy of the GNU General Public License     *");
	puts
		(" *   along with this program; if not, write to the                         *");
	puts
		(" *   Free Software Foundation, Inc.,                                       *");
	puts
		(" *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *");
	puts
		(" ***************************************************************************/\n");

	puts ("/* window.c - Blackman data windowing table (doubles in HEX) */");
	puts
		("/* Adjusted for windowing power loss, data scaling and FFTW normalization */");
	puts ("/* Automatically generated for `xspect' by Gordon Miller 2007 */");
	puts ("/* WARNING do not edit */\n");

/* First AND last points are ZERO ... */
	*fp1++ = *fp2-- = 0.0;
	stop = MAX_LEN / 2;

/* The main Blackman formula ... */
	for (i = 1; i < stop; i++, fp1++, fp2--)
	{
		*fp1 = *fp2 = BLACK1 - BRAISE * cos (2.0 * M_PI * i / (MAX_LEN - 1))
			+ BLACK2 * cos (4.0 * M_PI * i / (MAX_LEN - 1));

		sum += *fp1;
	}

/* Remedial gain factor ... */
	again = stop / sum / sqrt (MAX_LEN) / DSCALE;

/* No need to do first or last ... */
	fp1 = wind + 1;
	stop = MAX_LEN - 1;

/* Adjust the coefficients ... */
	for (i = 1; i < stop; i++, fp1++)
		*fp1 *= again;

/* Print the data definition ... */
	puts ("unsigned long long int hex_coefficients[] = {");

/* The main group ... */
	for (i = 0; i < count; i++, uip++)
	{
		if ((i % PERLINE) == 0)
			printf ("\n\t");

		printf (GFORMAT ", ", *uip);
	}

/* The last line ... */
	count = (MAX_LEN % PERLINE) - 1;
	printf ("\n\t");

	for (i = 0; i < count; i++, uip++)
		printf (GFORMAT ", ", *uip);

	printf (GFORMAT "\n", *uip);
	puts ("};");
	exit (EXIT_SUCCESS);

}																/* end main */
