/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* xevent.c - event processing for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
check_events (void)
/* Only called from dialog thread ... */
{
	XEvent event;

	while (!XPending (display))
	{
		if (quit_signal)
			dispatch_dialog (do_quit, DUMMY);

		else if (suspend_signal)
		{
			suspend_signal = 0;

			if (!freeze_mode)
				dispatch_dialog (do_toggle_freeze, DUMMY);
		}

		else if (resume_signal)
		{
			resume_signal = 0;

			if (freeze_mode)
				dispatch_dialog (do_toggle_freeze, DUMMY);
		}

		else
			snooze ();
	}

	XNextEvent (display, &event);

	if (event.type == ButtonPress)
	{
		if (help_on)
			close_help_window ();

		else if (about_on)
			close_about_window ();

		else if (scr_list_len)
		{
			if (event.xany.window == window)
			/* Off the menu tree ... */
				kill_all_menus ();

			else
			/* On the menu tree ... */
			{
				int count;
				int *ip = screen_list;

			/* Find the lady ... */
				for (count = 0; count < scr_list_len; count++, ip++)
					if (event.xany.window == menus[*ip].window)
						break;

			/* Process the menu event ... */
				perform_menu (*ip, &event);

			}													/* end else */
		}														/* end scr_list_len */

		else
		/* No clutter */
		{
			if (event.xbutton.button == Button3)
				draw_menu (0, event.xbutton.x, event.xbutton.y);

		}														/* end No clutter */
	}															/* end event.type == ButtonPress */

	else if (event.type == MotionNotify)
	/* On the menu tree ... */
	{
		int count;
		int *ip = screen_list;

	/* Find the lady ... */
		for (count = 0; count < scr_list_len; count++, ip++)
			if (event.xany.window == menus[*ip].window)
				break;

	/* Process the menu event ... */
		update_menu (*ip, &event);

	}															/* end event.type == MotionNotify */

	else if (event.type == KeyPress)
	{
		KeySym key_sym;
		char buffer[KEYLEN];

		if (help_on)
			close_help_window ();

		else if (about_on)
			close_about_window ();

		else
		/* Only (possibly) menu on ... */
		{
			if (scr_list_len)
				kill_all_menus ();

			XLookupString (&event.xkey, buffer, KEYLEN, &key_sym, NULL);
			process_key (key_sym);

		}
	}															/* end event.type == Keypress */

	else if (event.type == ClientMessage &&
					 event.xclient.message_type == wm_protocols &&
					 event.xclient.data.l[0] == wm_delete_window)

		dispatch_dialog (do_quit, DUMMY);

/* Peel out unwanted repeats */
	XSync (display, True);

}																/* end check_events */

/* vi: set ts=2: */
