/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* xkeyinput.c - keyboard input for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
process_key (int c)
{
/* Keys valid in both freeze mode and non-freeze mode */
	if (c == XK_Escape)
		return;

	else if (c == XK_F10)
		dispatch_dialog (do_quit, DUMMY);

	else if (c == XK_F1)
		show_help_window (DUMMY);

	else if (c == XK_F2)
		show_about_window (DUMMY);

	else if (c == XK_space)
		dispatch_dialog (do_toggle_freeze, DUMMY);

	else if (c == XK_g)
		dispatch_dialog (do_toggle_graticule, DUMMY);

	else if (c == XK_c)
		dispatch_dialog (do_toggle_rainbow, DUMMY);

	else if (c == XK_p)
		dispatch_dialog (do_toggle_peak, DUMMY);

	else if (c == XK_Down || c == XK_KP_2 || c == XK_KP_Down)
	{
		if (!y_zoom_bottom (DUMMY))
			dispatch_dialog (do_vertical_down, DUMMY);
	}

	else if (c == XK_Up || c == XK_KP_8 || c == XK_KP_Up)
	{
		if (!y_zoom_top (DUMMY))
			dispatch_dialog (do_vertical_up, DUMMY);
	}

	else if (c == XK_Page_Up || c == XK_KP_9 || c == XK_KP_Page_Up)
	{
		if (!y_zoom_off (DUMMY))
			dispatch_dialog (do_vertical_more, DUMMY);
	}

	else if (c == XK_Page_Down || c == XK_KP_3 || c == XK_KP_Page_Down)
	{
		if (!y_zoom_max (DUMMY))
			dispatch_dialog (do_vertical_less, DUMMY);
	}

	else if (c == XK_greater || c == XK_period)
	{
		if (!x_zoom_off (DUMMY))
			dispatch_dialog (do_horizontal_more, DUMMY);
	}

	else if (c == XK_less || c == XK_comma)
	{
		if (!x_zoom_max (DUMMY))
			dispatch_dialog (do_horizontal_less, DUMMY);
	}

	else if (c == XK_Right || c == XK_KP_Right)
	{
		if (!x_zoom_end (DUMMY))
			dispatch_dialog (do_horizontal_right, DUMMY);
	}

	else if (c == XK_Left || c == XK_KP_Left)
	{
		if (!x_zoom_start (DUMMY))
			dispatch_dialog (do_horizontal_left, DUMMY);
	}

	else if (c == XK_Home || c == XK_KP_Home)
	{
		if (!x_zoom_start (DUMMY))
			dispatch_dialog (do_horizontal_start, DUMMY);
	}

	else if (c == XK_End || c == XK_KP_End)
	{
		if (!x_zoom_end (DUMMY))
			dispatch_dialog (do_horizontal_end, DUMMY);
	}

	else if (c == XK_f)
	{
		if (!x_zoom_off (DUMMY))
			dispatch_dialog (do_horizontal_full, DUMMY);
	}

	else if (!freeze_mode)
	/* Keys only valid in non-freeze mode display ... */
	{
		if (c == XK_a)
			dispatch_dialog (do_toggle_averaging, DUMMY);

		else if (c == XK_o)
			dispatch_dialog (do_toggle_overlap, DUMMY);

		else if (c == XK_d)
			dispatch_dialog (do_callibrate_dc, DUMMY);

		else if (c == XK_R)
		{
			if (!rate_is (MAXINDEX))
				dispatch_dialog (do_rate_up, DUMMY);
		}

		else if (c == XK_b)
		{
			if (averaging_mode && !averaging_is (AVERMIN))
				dispatch_dialog (do_averaging_down, DUMMY);
		}

		else if (c == XK_B)
		{
			if (averaging_mode && !averaging_is (AVERMAX))
				dispatch_dialog (do_averaging_up, DUMMY);
		}

		else if (c == XK_v)
		{
			if (overlap_mode && !overlap_is (OVERMIN))
				dispatch_dialog (do_overlap_down, DUMMY);
		}

		else if (c == XK_V)
		{
			if (overlap_mode && !overlap_is (OVERMAX))
				dispatch_dialog (do_overlap_up, DUMMY);
		}

		else if (c == XK_r)
		{
			if (!rate_is (MININDEX))
				dispatch_dialog (do_rate_down, DUMMY);
		}

		else if (c == XK_S)
		{
			if (!channel_is (STEREOMAX))
				dispatch_dialog (do_stereo_up, DUMMY);
		}

		else if (c == XK_s)
		{
			if (!channel_is (STEREOMIN))
				dispatch_dialog (do_stereo_down, DUMMY);
		}
	}															/* end !freeze_mode */
}																/* end process_key */

/* vi: set ts=2: */
