/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* xmenu.c - menu for `xspect' by Gordon Miller 2007 */

#include "xspect.h"
#include "extern.h"

void
draw_menu (int menu_number, int x, int y)
/* Draw a (sub)menu ... within the dialog thread ... */
{
	Menu *mp = menus + menu_number;
	Menu_item *mip;
	int count;

	if (!mp->created)
	/* Create the new window ... */
	{
		XSizeHints *sizehints;

	/* Allocate space for hints */
		if ((sizehints = XAllocSizeHints ()) == 0)
			error ("Not enough memory for the size hints structure.");

	/* Create the window ... */
		mp->w = mp->list_max_len * FONT_WIDTH + 2 * (MHGAP + MHW);
		mp->h = mp->num_items * MSP + ML1 + 2 * MHW + 1;

		mp->window = XCreateSimpleWindow (display, window,
																			mp->x, mp->y, mp->w, mp->h, 0, 0, 0);

	/* Create the shadow pixmap ... */
		mp->shadow = XCreatePixmap (display, mp->window, HW, HH, depth);

	/* Make shadow pixmap the window background pixmap ... */
		XSetWindowBackgroundPixmap (display, mp->window, mp->shadow);

	/* Don't allow resize ... and place according to program */
		sizehints->flags = PPosition | PSize | PMinSize | PMaxSize;
		sizehints->min_width = mp->w;
		sizehints->min_height = mp->h;
		sizehints->max_width = mp->w;
		sizehints->max_height = mp->h;

		XSetStandardProperties (display, mp->window,
														NULL, NULL, 0, 0, 0, sizehints);

	/* Free up the hints space */
		XFree (sizehints);

	/* Menu background ... */
		XFillRectangle (display, mp->shadow, dialog_back_gc, 0, 0, mp->w, mp->h);

	/* First the menu highlight outline */
		XDrawLine (display, mp->shadow, dialog_hl_white_gc, 0, 0, mp->w - MHW, 0);
		XDrawLine (display, mp->shadow, dialog_hl_white_gc, 0, MHW, 0,
							 mp->h - MHW);

		XDrawLine (display, mp->shadow, dialog_hl_black_gc,
							 mp->w - MHW, MHW, mp->w - MHW, mp->h - MHW);

		XDrawLine (display, mp->shadow, dialog_hl_black_gc,
							 MHW, mp->h - MHW, mp->w - MHW, mp->h - MHW);

	/* Now the menu title */
		XDrawString (display, mp->shadow, dialog_title_gc, MTX, MTY, mp->title,
								 mp->title_len);

	/* A furrowed seperator */
		XDrawLine (display, mp->shadow, dialog_hl_black_gc, MHW, ML1,
							 mp->w - MHW - 1, ML1);

		XDrawLine (display, mp->shadow, dialog_hl_white_gc, MHW, ML2,
							 mp->w - MHW - 1, ML2);

	/* Look for motion  & button press events in the menu window ... */
		XSelectInput (display, mp->window, PointerMotionMask | ButtonPressMask);
		mp->created = 1;

	}															/* end if (!mp->created) */

	if (!mp->placed)
	{
		int max_x, max_y;

	/* Keep in the main window */
		max_x = W - mp->w - MGAP;
		mp->x = (x < max_x) ? x : max_x;
		max_y = H - mp->h - MGAP;
		mp->y = (y < max_y) ? y : max_y;
		XMoveWindow (display, mp->window, mp->x, mp->y);
		mp->placed = 1;
	}

/* Enter menu number onto list ... */
	mp->index = scr_list_len;
	screen_list[scr_list_len++] = menu_number;

/* Now the menu choices */
	mip = mp->menu_list;
	for (count = 0; count < mp->num_items; count++, mip++)
	{
		int dimmer = mip->dimmer ? mip->dimmer (mip->param) : 0;

		XDrawString (display, mp->shadow,
								 (dimmer ? dialog_dim_gc : dialog_fore_gc), MTX,
								 count * MSP + M1, mip->entry, mip->len);

	/* Draw the arrow, if any, passive */
		if (mip->type == TYPE_SUB)
		{
			if (dimmer)
				arrow (mp, count + 1, dialog_back_gc, dialog_back_gc, dialog_dim_gc);

			else
				arrow (mp, count + 1, dialog_hl_white_gc, dialog_hl_white_gc,
							 dialog_hl_black_gc);
		}
	}															/* end for */

/* Fire up the completed/reset window */
	XMapRaised (display, mp->window);

}																/* end draw_menu */

void
perform_menu (int menu_number, XEvent * eventp)
/* Process a found (sub)menu click event ... */
{
	Menu *mp = menus + menu_number;
	int count, this = 0;
	int by = eventp->xbutton.y;
	int num_items = mp->num_items;

/* Find the menu item ... */
	for (count = 1; count <= num_items; count++)
		if (by >= MA1 + (count - 1) * MSP && by < MA1 + count * MSP)
		{
			this = count;
			break;
		}

	if (this == 0)
	/* Menu title chosen ... */
		kill_all_menus ();

	else
	/* Menu item chosen ... */
	{
		Menu_item *mip = mp->menu_list + this - 1;
		int dimmer = mip->dimmer ? mip->dimmer (mip->param) : 0;
		int type = mip->type;

		if (this && !dimmer && (type != TYPE_SUB))
		{
			kill_all_menus ();

			if (type == TYPE_CALL)
				dispatch_dialog (mip->action, mip->param);

			else if (type == TYPE_DIAL)
				mip->action (DUMMY);
		}
	}															/* end else chosen */
}																/* end perform_menu */

void
update_menu (int menu_number, XEvent * eventp)
/* Process a found (sub)menu motion event ... */
{
	Menu *mp = menus + menu_number;
	int count, this = 0;
	int my = eventp->xmotion.y;
	int previous = mp->previous;
	int num_items = mp->num_items;

	for (count = 1; count <= num_items; count++)
		if (my >= MA1 + (count - 1) * MSP && my < MA1 + count * MSP)
		{
			this = count;
			break;
		}

	if (this != previous)
	/* A new menu item ... */
	{
		Menu_item *mip;
		int dimmer;

		if (previous)
		/* Unhighlight previous item */
		{
			mip = mp->menu_list + previous - 1;
			dimmer = mip->dimmer ? mip->dimmer (mip->param) : 0;

			if (!dimmer)
			{
				if (mip->type == TYPE_SUB)
				/* Deactivate sub_menu, if any ... */
				{
					arrow (mp, previous, dialog_hl_white_gc, dialog_hl_white_gc,
								 dialog_hl_black_gc);

					kill_menus (mp->index + 1);
				}

				unhighlight (mp, previous);

			}													/* end if !dimmer */
		}														/* end if previous */

		if (this)
		/* Highlight current item */
		{
			mip = mp->menu_list + this - 1;
			dimmer = mip->dimmer ? mip->dimmer (mip->param) : 0;

			if (!dimmer)
			{
				if (mip->type == TYPE_SUB)
				/* Activate sub_menu, if any ... */
				{
					arrow (mp, this, dialog_hl_black_gc, dialog_hl_black_gc,
								 dialog_hl_white_gc);

					draw_menu (mip->sub_menu, mp->x + mp->w,
										 mp->y + MA1 + (this - 1) * MSP);
				}

				highlight (mp, this);

			}													/* end if !dimmer */
		}														/* end if this */

		mp->previous = this;

	/* Fire up the updated window */
		XClearWindow (display, mp->window);

	}															/* end this != mp->previous */
}																/* end update_menu */

void
highlight (Menu * mp, int entry)
/* Highlight a menu item ... */
{
	XDrawLine (display, mp->shadow, dialog_hl_white_gc,
						 2 * MHW - 1, MA1 + (entry - 1) * MSP,
						 mp->w - 2 * MHW, MA1 + (entry - 1) * MSP);

	XDrawLine (display, mp->shadow, dialog_hl_white_gc,
						 2 * MHW - 1, MA1 + (entry - 1) * MSP,
						 2 * MHW - 1, MA1 + entry * MSP - 1);

	XDrawLine (display, mp->shadow, dialog_hl_black_gc,
						 mp->w - 2 * MHW, MA1 + (entry - 1) * MSP,
						 mp->w - 2 * MHW, MA1 + entry * MSP - 1);

	XDrawLine (display, mp->shadow, dialog_hl_black_gc,
						 2 * MHW, MA1 + entry * MSP - 1,
						 mp->w - 2 * MHW, MA1 + entry * MSP - 1);

}																/* end highlight */

void
unhighlight (Menu * mp, int entry)
/* Unhighlight a menu item ... */
{
	XDrawRectangle (display, mp->shadow, dialog_back_gc,
									2 * MHW - 1, MA1 + (entry - 1) * MSP,
									mp->w - 2 * MHW - 1, MSP - 1);

}																/* end unhighlight */

void
arrow (Menu * mp, int entry, GC first, GC second, GC third)
/* Draw an arrow to indicate sub_menu ... */
{
/* These will be optimized away ... */
	int x1 = MTX + (mp->list_max_len - 1) * (FONT_WIDTH);
	int y1 = (entry - 1) * MSP + M1 - 1;
	int x2 = x1;
	int y2 = y1 - 7;
	int x3 = x1 + 7;
	int y3 = y1 - 4;

/* A triangle in 2 colours ... */
	XDrawLine (display, mp->shadow, first, x1, y1, x2, y2);
	XDrawLine (display, mp->shadow, second, x2, y2, x3, y3);
	XDrawLine (display, mp->shadow, third, x3, y3, x1, y1);

}																/* end arrow */

void
kill_menus (int start_menu)
/* Kill off the tail of the menu tree ... */
{
	int count, len = scr_list_len;
	int *ip = screen_list + start_menu;

	for (count = start_menu; count < len; count++, ip++)
	{
		Menu *mp = menus + (*ip);
		int previous = mp->previous;

		if (previous)
		/* Clean up menu windows ... */
		{
			Menu_item *mip = mp->menu_list + previous - 1;
			int dimmer = mip->dimmer ? mip->dimmer (mip->param) : 0;

			if (!dimmer)
				unhighlight (mp, previous);

			mp->previous = 0;
		}

		XUnmapWindow (display, mp->window);
		scr_list_len--;

	}															/* end for */
}																/* end kill_menus */

void
kill_all_menus (void)
/* Kill off the entire menu tree ... */
{
	int count;
	int *ip = screen_list;
	Menu *mp;

	for (count = 0; count < scr_list_len; count++, ip++)
	{
		int previous;

		mp = menus + (*ip);
		previous = mp->previous;

		if (previous)
		/* Clean up menu windows ... */
		{
			Menu_item *mip = mp->menu_list + previous - 1;
			int dimmer = mip->dimmer ? mip->dimmer (mip->param) : 0;

			if (!dimmer)
				unhighlight (mp, previous);

			mp->previous = 0;
		}

		XUnmapWindow (display, mp->window);

	}															/* end for */

	for (mp = menus, count = 0; count < MENU_LENGTH; count++, mp++)
		mp->placed = 0;

	scr_list_len = 0;

}																/* end kill_all_menus */

/* vi: set ts=2: */
