/***************************************************************************
 *   Copyright (C) 2007 by Gordon Miller                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* xspect.h - includes & defines for `xspect' by Gordon Miller 2007 */

#include <ctype.h>
#include <fcntl.h>
#include <getopt.h>
#include <math.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <pthread.h>
#include <fftw3.h>

#include <alsa/asoundlib.h>
#include <sys/types.h>
#include <sys/time.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#define	PROGNAME		"xspect"
#define	VERSION 		"v1.01"
#define DEF_INI_FILE	"/.xspect.ini"
#define DUMMY		(0)
#define KEYLEN		(20)

#define	WNAME	PROGNAME " - DSP"	/* Window title */

/* FFTW stuff ... */
#define MYWISDOM	"/.xspect.wisdom"
#define MYFLAGS		(FFTW_MEASURE)
#define MYDIREC		(FFTW_R2HC)

/* Maximum buffer length */
#define	MAX_LEN		(1024)
#define	PER_LEN		(64)
#define BPSMONO		(2)
#define BPSSTEREO	(2 * BPSMONO)
#define ALSA_LEN	(80)

/* Minimum & maximum sampling rate indexes */
#define	MININDEX	(0)
#define	MAXINDEX	(6)

/* Minimum & maximum averaging factor */
#define AVERMIN		(10)
#define AVERMAX 	(90)
#define AVERSTEP	(10)

/* Minimum & maximum overlap factor */
#define OVERMIN	(2)
#define OVERMAX (9)

/* Minimum & maximum log labels */
#define LOGBASEMIN	(-140)
#define LOGBASEMAX	(-10)
#define LOGTOPMIN		(-130)
#define LOGTOPMAX		(0)
#define LOGMIN			(10)

/* Bin stuff */
#define BINMIN		(0)
#define BINMAX		(WIDE)
#define NBINMIN		(16)

/* X Zoom */
#define XZOOM			(2)
#define XOVER			(4)
#define MAXZOOM		(BINMAX/NBINMIN)

/* Reset peak */
#define RESETF		(-1.0)
#define RESETDB		(1.0)
#define LOWPEAK		(-139.9)

/* Minimum & maximum stereo modes */
#define STEREOMIN	(0)
#define	STEREOMAX	(2)

/* Minimum & maximum DC offset */
#define	MINDC		(-1024)
#define	MAXDC		(1024)

/* Data buffers ... */
#define	BUFFERS	(2)

/* Ticks per second for `snooze' */
#define TPS			(1000000.0)

/* Ordinary palette colour range ... */
#define PALMIN	(0)
#define PALMAX	(63)

/* The following fonts are fixed */
/*
#define	FONT1				"lucidasanstypewriter-14"
*/
#define	FONT1				"9x15"
#define	FONT2				"9x15"
#define FONT_HEIGHT (13)
#define FONT_UP			(11)
#define FONT_WIDTH  (9)

/* Define the positions of the various display elements */
#define	WINDOW_TOP		(57)			/* The spectrum box */
#define	WINDOW_BOTTOM	(385)
#define	WINDOW_LEFT		(64)
#define	WINDOW_RIGHT	(575)
#define	HIGH					(WINDOW_BOTTOM-WINDOW_TOP+1)
#define	WIDE					(WINDOW_RIGHT-WINDOW_LEFT+1)
#define XSTEP					(32)

#define	X							(0)				/* Window x position */
#define	Y							(0)				/* Window y position */
#define	W							(640)			/* Window width */
#define	H							(480)			/* Window height */

#define	YSP	(16)								/* Header line spacing */
#define	COL	(21*FONT_WIDTH)			/* Jump to second column */

#define	SRX	(WINDOW_LEFT)				/* Sampling Rate */
#define	SRY	(16)

#define OVX	(SRX)								/* Overlap */
#define OVY	(SRY + YSP)

#define FSX	(SRX)								/* Frame rate */
#define FSY	(OVY + YSP)

#define	AVX	(SRX + COL)					/* Averaging */
#define	AVY	(SRY)

#define	SMX	(AVX)								/* Stereo mode */
#define	SMY	(AVY + YSP)

#define ZMX	(AVX)								/* X Zoom */
#define ZMY	(SMY + YSP)

#define	DCX	(AVX + COL)					/* DC offset */
#define	DCY	(AVY)

#define FRX	(DCX)								/* Freeze */
#define FRY	(DCY + YSP)

#define RAX	(DCX)								/* Rainbow */
#define RAY	(FRY + YSP)

#define	PKX	(WINDOW_LEFT)				/* Peak */
#define	PKY	(474)

/* Colour triplet */
typedef struct Rgb
{
	int red;
	int green;
	int blue;
}
Rgb;

/* Menu elements */
#define	MHW		(1)								/* Menu highlight width */
#define	MGAP	(2)								/* Gap to edge of main window */

#define	MVGAP		(2)							/* Menu vertical gap */
#define	MHGAP		(2)							/* Menu horiz gap */

#define	MTX		(MHW+MHGAP-1)			/* Title x */
#define	MTY		(MHW+MVGAP+FONT_UP)	/* Title y */
#define	ML1		(MHW+2*MVGAP+FONT_HEIGHT)	/* First line y */
#define	ML2		(ML1+1)						/* Second line y */
#define	MA1		(ML2+1)						/* First entry highlight y */
#define	M1		(MA1+MHW+MVGAP+FONT_UP)	/* First entry text y */
#define	MSP		(FONT_HEIGHT+2*MHW+2*MVGAP)	/* Entry y spacing */

#define MAX_DEPTH			(8)				/* Maximum menu depth */
#define MENU_LENGTH		(14)			/* Number of (sub)menus */

#define TYPE_CALL			(0)
#define TYPE_SUB			(1)
#define TYPE_DIAL			(2)
#define TYPE_NULL			(3)

typedef struct Menu_item
{
/* Required at creation time ... initialize as appropriate */
	char *entry;									/* Text of menu choice */
	int len;											/* Length of above */
	int type;											/* Callback, submenu, dialog or null */
	void (*action) (int);					/* What to do */
	int param;										/* Parameter */
	int (*dimmer) (int);					/* Dimmer function */
	int sub_menu;									/* Submenu number */
}
Menu_item;

typedef struct Menu
{
/* Required at creation time ... initialize as appropriate */
	char *title;									/* Text of menu title */
	int title_len;								/* Length of above */
	Menu_item *menu_list;					/* The list of entries */
	int num_items;								/* Number of items */

/* Add two to menu entries with sub_menu to get maxentry length ... */
	int list_max_len;							/* Maximum length of the entries/title */

/* These will be filled in later ... initialize with zeroes */
	Window window;								/* Window id & placement */
	Pixmap shadow;								/* Window shadow pixmap */
	int x;
	int y;
	int w;
	int h;
	int created;									/* Are we created ? */
	int placed;										/* Are we placed ? */
	int previous;									/* Previous menu item */
	int index;										/* Index in active menu window list ... */
}
Menu;

/* Help window ... */
#define HELP_TOTAL1	(15)
#define HELP_TOTAL2	(15)
#define HELP_KEYMAX	(5)
#define HELP_DESMAX	(16)
#define HELP_KEYW		(HELP_KEYMAX*FONT_WIDTH)
#define HELP_DESW		(HELP_DESMAX*FONT_WIDTH)
#define HELP_ENTGAP	(2*FONT_WIDTH)
#define HELP_ENTMAX	(HELP_KEYW + HELP_DESW + HELP_ENTGAP)
#define HCG					(3*FONT_WIDTH)
#define HC1K				(MHW + FONT_WIDTH)
#define HC1D				(HC1K + HELP_KEYW + HELP_ENTGAP)
#define HC2K				(HC1D + HELP_DESW + HCG)
#define HC2D				(HC2K + HELP_KEYW + HELP_ENTGAP)

#define HT		"Keys"
#define HTL		(4)
#define HW		(2*HELP_ENTMAX + HCG + 2*(FONT_WIDTH + MHW))
#define HH		(HELP_TOTAL1*MSP + ML1 + 2*MHW + 1)
#define HX		((W - HW)/2)
#define HY		((HIGH - HH)/2 + WINDOW_TOP)
#define HTX		((HW - FONT_WIDTH*HTL)/2)
#define HTY		(MTY)

typedef struct Help_item
{
/* Required at creation time ... initialize as appropriate */
	char *key;										/* The key label */
	int key_len;									/* Length of above */
	char *description;						/* What the key does */
	int description_len;					/* Length of above */
}
Help_item;

/* About window ... */
#define AT		"About"
#define ATL		(5)
#define AM		PROGNAME " - by Gordon Miller (C) & (A) 2007"
#define AML		(40)
#define AW		((AML + 2)*FONT_WIDTH + 2*(MHGAP + MHW))
#define AH		(MSP + ML1 + 2*MHW + 1)
#define AX		((W - AW)/2)
#define AY		((HIGH - AH)/2 + WINDOW_TOP)
#define ATX		((AW - FONT_WIDTH*ATL)/2)
#define ATY		(MTY)
#define AMX		(MTX + FONT_WIDTH)
#define AMY		(M1)

/* vi: set ts=2: */
